@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -110,7 +110,6 @@ static ErrorStrings Errors[] =
     {ERR_LINK_EXIST, "POP3 link already exist"},
     {ERR_WRITE_LINKS_FILE, "Error writing POP3 links file"},
     {ERR_LINK_NOT_FOUND, "POP3 link not found"},
-    {ERR_NO_SMTP_SPOOL_FILES, "No SMTP spool files ready to process"},
     {ERR_SMTPGW_FILE_NOT_FOUND, "SMTP gateway file not found"},
     {ERR_GATEWAY_ALREADY_EXIST, "SMTP gateway already exist"},
     {ERR_GATEWAY_NOT_FOUND, "SMTP gateway not found"},
@@ -120,7 +119,6 @@ static ErrorStrings Errors[] =
     {ERR_MLUSER_NOT_FOUND, "Mailing list user not found"},
     {ERR_SPOOL_FILE_NOT_FOUND, "Spool file not found"},
     {ERR_INVALID_SPOOL_FILE, "Invalid spool file"},
-    {ERR_SPOOL_FILE_NOT_READY, "Spool file not ready"},
     {ERR_SPOOL_FILE_EXPIRED, "Spool file has reached max retry ops"},
     {ERR_SMTPRELAY_FILE_NOT_FOUND, "SMTP relay file not found"},
     {ERR_DOMAINS_FILE_NOT_FOUND, "POP3 domains file not found"},
@@ -172,7 +170,6 @@ static ErrorStrings Errors[] =
     {ERR_DOMAIN_ALREADY_HANDLED, "Domain already exist"},
     {ERR_BAD_CTRL_LOGIN, "Bad controller login"},
     {ERR_CTRL_ACCOUNTS_FILE_NOT_FOUND, "Controller accounts file not found"},
-    {ERR_RBL_SPAMMER, "RBL registered spammer (rbl.maps.vix.com.)"},
     {ERR_SPAMMER_IP, "Server registered spammer IP"},
     {ERR_TRUNCATED_DGRAM_DNS_RESPONSE, "Truncated UDP DNS response"},
     {ERR_NO_DGRAM_DNS_RESPONSE, "Unable to get UDP DNS response"},
@@ -185,7 +182,6 @@ static ErrorStrings Errors[] =
     {ERR_NO_MAILPROC_FILE, "User mail processing file not present"},
     {ERR_DNS_RECURSION_NOT_AVAILABLE, "DNS recursion not available"},
     {ERR_POP3_EXTERNAL_LINK_DISABLED, "External POP3 link disabled"},
-    {ERR_RSS_SPAMMER, "RSS registered spammer (relays.mail-abuse.org.)"},
     {ERR_BAD_DOMAIN_PROC_CMD_SYNTAX, "Error in custom domain processing file syntax"},
     {ERR_NOT_A_CUSTOM_DOMAIN, "Not a custom domain"},
     {ERR_NO_MORE_TOKENS, "No more tokens"},
@@ -242,9 +238,6 @@ static ErrorStrings Errors[] =
     {ERR_SMTPFWD_NOT_FOUND, "SMTP forward gateway not found"},
     {ERR_USER_BREAK, "Operation interrupted"},
     {ERR_SET_THREAD_PRIORITY, "Error setting thread priority"},
-    {ERR_INVALID_QUEUE_PATH, "Invalid queue path"},
-    {ERR_QUEUE_ARRAY_FULL, "Queue array is full"},
-    {ERR_QUEUE_ENTRY_NOT_FOUND, "Queue entry not found"},
     {ERR_NULL_SENDER, "Empty message sender"},
     {ERR_RCPTTO_UNKNOWN, "Mail tag \"To:\" missing"},
     {ERR_LOADMODULE, "Error moading dynamic module"},
@@ -258,10 +251,33 @@ static ErrorStrings Errors[] =
     {ERR_THREADCREATE, "Error in function {pthread_create}"},
     {ERR_CREATEMUTEX, "Error in function {CreateMutex}"},
     {ERR_NO_LOCAL_SPOOL_FILES, "Local spool empty"},
-    {ERR_NO_POP3_DOMAIN, "Unable to retrieve a valid POP3 domain from peer IP"},
+    {ERR_NO_HANDLED_DOMAIN, "Unable to retrieve an handled domain from peer IP"},
     {ERR_INVALID_MAIL_DOMAIN, "Remote domain has no DNS/MX entries"},
     {ERR_BAD_CMDSTR_CHARS, "Bad characters in command line"},
     {ERR_FETCHMSG_UNDELIVERED, "POP3 fetched message failed delivery"},
+    {ERR_USER_VAR_NOT_FOUND, "User configuration variable not found"},
+    {ERR_NO_POP3_IP, "Invalid or not available POP3 connection IP"},
+    {ERR_NO_MESSAGE_FILE, "Message file not existent"},
+    {ERR_GET_DISK_SPACE_INFO, "Error getting disk space info"},
+    {ERR_GET_MEMORY_INFO, "Error getting memory info"},
+    {ERR_LOW_DISK_SPACE, "System low in disk space"},
+    {ERR_LOW_VM_SPACE, "System low in virtual memory"},
+    {ERR_USER_DISABLED, "Account disabled"},
+    {ERR_BAD_DNS_NAME_RECORD, "Bad format for DNS name record"},
+    {ERR_MESSAGE_SIZE, "Message exceeds fixed maximum message size"},
+    {ERR_SMTPSRV_MSG_SIZE, "Message too big for the remote SMTP server"},
+    {ERR_MAPS_CONTAINED, "The peer IP is mapped"},
+    {ERR_ADOMAIN_FILE_NOT_FOUND, "Domain aliases file not found"},
+    {ERR_ADOMAIN_EXIST, "Domain alias already exist"},
+    {ERR_ADOMAIN_NOT_FOUND, "Domain alias not found"},
+    {ERR_NOT_A_CMD_ALIAS, "Cmd alias not found"},
+    {ERR_GETSOCKOPT, "Error in function {getsockopt}"},
+    {ERR_NO_HDR_FETCH_TAGS, "No fetch headers tags string supplied"},
+    {ERR_SET_FILE_TIME, "Error setting file times"},
+    {ERR_LISTDIR_NOT_FOUND, "Listing directory not found"},
+    {ERR_DUPLICATE_HANDLE, "Error in function {DuplicateHandle}"},
+    {ERR_EMPTY_LOG, "Log file is empty"},
+    {ERR_BAD_RELATIVE_PATH, "Error in relative path syntax"},
 
 };
 
