@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include "MiscUtils.h"
 #include "ResLocks.h"
 #include "BuffSock.h"
-#include "Queue.h"
+#include "MessQueue.h"
 #include "SvrUtils.h"
 #include "UsrUtils.h"
 #include "SMTPUtils.h"
@@ -52,7 +52,6 @@
 
 #define LMAIL_SERVER_NAME           "[" APP_NAME_VERSION_OS_STR " LMAIL Server]"
 #define LOCAL_SPOOL_DIR             "local"
-#define STD_LMAILTHREAD_SLEEP_TIME  4
 #define LMAIL_LOG_FILE              "lmail"
 
 
@@ -95,14 +94,14 @@ static int      LMAILLogMessage(char const * pszMailFile, char const * pszSMTPDo
 
 
 
-char           *LMAILGetSpoolDir(char *pszSpoolPath)
+char           *LMAILGetSpoolDir(char *pszSpoolPath, int iMaxPath)
 {
 
-    SvrGetSpoolDir(pszSpoolPath);
+    SvrGetSpoolDir(pszSpoolPath, iMaxPath);
 
     AppendSlash(pszSpoolPath);
 
-    strcat(pszSpoolPath, LOCAL_SPOOL_DIR);
+    StrNCat(pszSpoolPath, LOCAL_SPOOL_DIR, iMaxPath);
 
     return (pszSpoolPath);
 
@@ -193,8 +192,9 @@ unsigned int    LMAILThreadProc(void *pThreadData)
     }
 
 ///////////////////////////////////////////////////////////////////////////////
-//  Get thread id
+//  Get thread id and sleep timeout
 ///////////////////////////////////////////////////////////////////////////////
+    int             iSleepTimeout = pLMAILCfg->iSleepTimeout;
     long            lThreadId = pLMAILCfg->lThreadCount;
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -231,7 +231,7 @@ unsigned int    LMAILThreadProc(void *pThreadData)
         int             iProcessResult = LMAILProcessLocalSpool(hShbLMAIL, lThreadId);
 
         if (iProcessResult == ERR_NO_LOCAL_SPOOL_FILES)
-            SysSleep(STD_LMAILTHREAD_SLEEP_TIME);
+            SysSleep(iSleepTimeout);
 
     }
 
@@ -295,14 +295,15 @@ static int      LMAILGetFilesSnapShot(LMAILConfig * pLMAILCfg, long lThreadId,
 
     char            szSpoolDir[SYS_MAX_PATH] = "";
 
-    LMAILGetSpoolDir(szSpoolDir);
+    LMAILGetSpoolDir(szSpoolDir, sizeof(szSpoolDir));
 
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Share lock local spool directory
 ///////////////////////////////////////////////////////////////////////////////
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szSpoolDir, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szSpoolDir, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -368,14 +369,15 @@ static int      LMAILRemoveProcessed(LMAILConfig * pLMAILCfg, char const * pszLi
 
     char            szSpoolDir[SYS_MAX_PATH] = "";
 
-    LMAILGetSpoolDir(szSpoolDir);
+    LMAILGetSpoolDir(szSpoolDir, sizeof(szSpoolDir));
 
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Share lock local spool directory
 ///////////////////////////////////////////////////////////////////////////////
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szSpoolDir, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szSpoolDir, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -417,7 +419,7 @@ static int      LMAILProcessList(LMAILConfig * pLMAILCfg, long lThreadId,
 
     char            szSpoolDir[SYS_MAX_PATH] = "";
 
-    LMAILGetSpoolDir(szSpoolDir);
+    LMAILGetSpoolDir(szSpoolDir, sizeof(szSpoolDir));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Retrieve SMTP domain
@@ -456,6 +458,14 @@ static int      LMAILProcessList(LMAILConfig * pLMAILCfg, long lThreadId,
 
         if (LMAILSubmitLocalFile(pLMAILCfg, szSpoolFilePath, lThreadId, szSMTPDomain) < 0)
         {
+            SysLogMessage(LOG_LEV_ERROR, "LMAIL [%02ld] error ( \"%s\" ): %s\n",
+                    lThreadId, ErrGetErrorString(), szSpoolFilePath);
+
+        }
+        else
+        {
+            SysLogMessage(LOG_LEV_MESSAGE, "LMAIL [%02ld] file processed: %s\n",
+                    lThreadId, szSpoolFilePath);
 
         }
 
@@ -477,7 +487,7 @@ static int      LMAILSubmitLocalFile(LMAILConfig * pLMAILCfg, const char *pszMai
 
     if (pMailFile == NULL)
     {
-        ErrSetErrorCode(ERR_FILE_OPEN, pszMailFile);
+        ErrSetErrorCode(ERR_FILE_OPEN);
         return (ERR_FILE_OPEN);
     }
 
@@ -553,23 +563,28 @@ static int      LMAILSubmitLocalFile(LMAILConfig * pLMAILCfg, const char *pszMai
             !IsEmptyString(szSpoolLine))
     {
 ///////////////////////////////////////////////////////////////////////////////
-//  Get unique spool/tmp file path
+//  Get message handle
 ///////////////////////////////////////////////////////////////////////////////
-        char            szSpoolTmpFile[SYS_MAX_PATH] = "";
+        QMSG_HANDLE     hMessage = QueCreateMessage(hSpoolQueue);
 
-        if (QueGetTempFile(NULL, szSpoolTmpFile, iQueueSplitLevel) < 0)
+        if (hMessage == INVALID_QMSG_HANDLE)
         {
             ErrorPush();
             fclose(pMailFile);
             return (ErrorPop());
         }
 
+        char            szQueueFilePath[SYS_MAX_PATH] = "";
 
-        FILE           *pSpoolFile = fopen(szSpoolTmpFile, "wb");
+        QueGetFilePath(hSpoolQueue, hMessage, szQueueFilePath);
+
+
+        FILE           *pSpoolFile = fopen(szQueueFilePath, "wb");
 
         if (pSpoolFile == NULL)
         {
-            CheckRemoveFile(szSpoolTmpFile);
+            QueCleanupMessage(hSpoolQueue, hMessage);
+            QueCloseMessage(hSpoolQueue, hMessage);
             fclose(pMailFile);
             ErrSetErrorCode(ERR_FILE_CREATE);
             return (ERR_FILE_CREATE);
@@ -615,23 +630,42 @@ static int      LMAILSubmitLocalFile(LMAILConfig * pLMAILCfg, const char *pszMai
         {
             ErrorPush();
             fclose(pSpoolFile);
+            QueCleanupMessage(hSpoolQueue, hMessage);
+            QueCloseMessage(hSpoolQueue, hMessage);
             fclose(pMailFile);
-            SysRemove(szSpoolTmpFile);
             return (ErrorPop());
         }
 
-        fclose(pSpoolFile);
+        if (SysFileSync(pSpoolFile) < 0)
+        {
+            ErrorPush();
+            fclose(pSpoolFile);
+            QueCleanupMessage(hSpoolQueue, hMessage);
+            QueCloseMessage(hSpoolQueue, hMessage);
+            fclose(pMailFile);
+            return (ErrorPop());
+        }
+
+        if (fclose(pSpoolFile))
+        {
+            QueCleanupMessage(hSpoolQueue, hMessage);
+            QueCloseMessage(hSpoolQueue, hMessage);
+            fclose(pMailFile);
+            ErrSetErrorCode(ERR_FILE_WRITE, szQueueFilePath);
+            return (ERR_FILE_WRITE);
+        }
 
         fseek(pMailFile, ulCurrOffset, SEEK_SET);
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Transfer file to the spool
 ///////////////////////////////////////////////////////////////////////////////
-        if (QueCommitTempMessage(szSpoolTmpFile) < 0)
+        if (QueCommitMessage(hSpoolQueue, hMessage) < 0)
         {
             ErrorPush();
+            QueCleanupMessage(hSpoolQueue, hMessage);
+            QueCloseMessage(hSpoolQueue, hMessage);
             fclose(pMailFile);
-            SysRemove(szSpoolTmpFile);
             return (ErrorPop());
         }
     }
