@@ -1,6 +1,6 @@
 /*
- *  MailSvr by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999  Davide Libenzi
+ *  XMail by Davide Libenzi ( Intranet and Internet mail server )
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Davide Libenzi <davide_libenzi@mycio.com>
+ *  Davide Libenzi <davidel@xmailserver.org>
  *
  */
 
@@ -27,9 +27,9 @@
 
 
 
-char           *CfgGetRootPath(char *pszPath);
-char           *CfgGetBasedPath(char const * pszFullPath, char *pszBasePath);
-char           *CfgGetFullPath(char const * pszRelativePath, char *pszFullPath);
+char           *CfgGetRootPath(char *pszPath, int iMaxPath);
+char           *CfgGetBasedPath(char const * pszFullPath, char *pszBasePath, int iMaxPath);
+char           *CfgGetFullPath(char const * pszRelativePath, char *pszFullPath, int iMaxPath);
 
 
 
