@@ -1,6 +1,6 @@
 /*
- *  MailSvr by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999  Davide Libenzi
+ *  XMail by Davide Libenzi ( Intranet and Internet mail server )
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Davide Libenzi <davide_libenzi@mycio.com>
+ *  Davide Libenzi <davidel@xmailserver.org>
  *
  */
 
@@ -39,12 +39,16 @@
 #include "DNSCache.h"
 #include "UsrUtils.h"
 #include "SvrUtils.h"
-#include "Queue.h"
+#include "MessQueue.h"
+#include "SMAILUtils.h"
+#include "QueueUtils.h"
 #include "ExtAliases.h"
+#include "AliasDomain.h"
 #include "MailDomains.h"
 #include "POP3GwLink.h"
 #include "CTRLSvr.h"
 #include "FINGSvr.h"
+#include "LMAILSvr.h"
 #include "DynDNS.h"
 #include "AppDefines.h"
 #include "MailSvr.h"
@@ -58,7 +62,7 @@
 #define ENV_CMD_LINE                "MAIL_CMD_LINE"
 #define SVR_SHUTDOWN_FILE           ".shutdown"
 #define STD_SMAIL_THREADS           16
-#define MAX_SMAIL_THREADS           64
+#define MAX_SMAIL_THREADS           256
 #define STD_SMAIL_RETRY_TIMEOUT     480
 #define STD_SMAIL_RETRY_INCR_RATIO  16
 #define STD_SMAIL_MAX_RETRY         32
@@ -66,17 +70,20 @@
 #define STD_PSYNC_NUM_THREADS       8
 #define MAX_PSYNC_NUM_THREADS       32
 #define STD_POP3_BADLOGIN_WAIT      5
-#define MAX_POP3_THREADS            512
-#define MAX_SMTP_THREADS            512
+#define MAX_POP3_THREADS            1024
+#define MAX_SMTP_THREADS            1024
 #define STD_SMTP_MAX_RCPTS          100
 #define MAX_CTRL_THREADS            512
-#define MAIL_MAGIC_KEY              0xabcd
+#define STD_LMAIL_THREADS           3
+#define MAX_LMAIL_THREADS           17
+#define STD_LMAILTHREAD_SLEEP_TIME  2
 #define SVR_EXIT_WAIT               480
 #define STD_SERVER_SESSION_TIMEOUT  90
 #define MAX_CLIENTS_WAIT            300
 #define CTRL_SERVER_SESSION_TIMEOUT 120
 #define SERVER_SLEEP_TIMESLICE      2
 #define SHUTDOWN_CHECK_TIME         2
+#define STD_POP3AUTH_EXPIRE_TIME    (15 * 60)
 
 
 
@@ -101,6 +108,8 @@ static int      SvrSetupSMAIL(int iArgCount, char *pszArgs[]);
 static void     SvrCleanupSMAIL(void);
 static int      SvrSetupPSYNC(int iArgCount, char *pszArgs[]);
 static void     SvrCleanupPSYNC(void);
+static int      SvrSetupLMAIL(int iArgCount, char *pszArgs[]);
+static void     SvrCleanupLMAIL(void);
 static int      SvrSetup(int iArgCount, char *pszArgs[]);
 static void     SvrCleanup(void);
 static void     SvrBreakHandler(void);
@@ -115,37 +124,33 @@ static char   **SvrMergeArgs(int iArgs, char *pszArgs[], int &iArgsCount);
 ///////////////////////////////////////////////////////////////////////////////
 //  External visible variabiles
 ///////////////////////////////////////////////////////////////////////////////
-SharedBlock     SHB_CTRLSvr,
-                SHB_FINGSvr,
-                SHB_SMTPSvr,
-                SHB_SMAILSvr,
-                SHB_SMAILQueue,
-                SHB_POP3Svr,
-                SHB_PSYNCSvr;
+SHB_HANDLE      hShbFING,
+                hShbCTRL,
+                hShbPOP3,
+                hShbSMTP,
+                hShbSMAIL,
+                hShbPSYNC,
+                hShbLMAIL;
 char            szMailPath[SYS_MAX_PATH];
-unsigned int    uKeyMagic = MAIL_MAGIC_KEY;
-SYS_IPCNAME     SemPSYNCThreads;
+QUEUE_HANDLE    hSpoolQueue;
+SYS_SEMAPHORE   hSyncSem;
 bool            bServerDebug;
-int             iLogRotateDays = LOG_ROTATE_DAYS;
+int             iLogRotateHours = LOG_ROTATE_HOURS;
 int             iQueueSplitLevel = STD_QUEUEFS_DIRS_X_LEVEL;
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Local visible variabiles
 ///////////////////////////////////////////////////////////////////////////////
 static char     szShutdownFile[SYS_MAX_PATH];
-static SHB_HANDLE hShbFING,
-                hShbCTRL,
-                hShbPOP3,
-                hShbSMTP,
-                hShbSMAIL,
-                hShbPSYNC;
+static bool     bServerShutdown = false;
 static int      iNumSMAILThreads;
-static SYS_SEMAPHORE SemSyncID;
+static int      iNumLMAILThreads;
 static SYS_THREAD hCTRLThread,
                 hFINGThread,
                 hPOP3Thread,
                 hSMTPThread,
                 hSMAILThreads[MAX_SMAIL_THREADS],
+                hLMAILThreads[MAX_LMAIL_THREADS],
                 hPSYNCThread;
 
 
@@ -161,6 +166,8 @@ static void     SvrShutdownCleanup(void)
 
     CheckRemoveFile(szShutdownFile);
 
+    bServerShutdown = false;
+
 }
 
 
@@ -168,6 +175,13 @@ static void     SvrShutdownCleanup(void)
 
 static int      SvrSetShutdown(void)
 {
+///////////////////////////////////////////////////////////////////////////////
+//  Set the shutdown flag and shutdown the library
+///////////////////////////////////////////////////////////////////////////////
+    bServerShutdown = true;
+
+    SysShutdownLibrary();
+
 
     FILE           *pFile = fopen(szShutdownFile, "wt");
 
@@ -236,23 +250,15 @@ static int      SvrSetupCTRL(int iArgCount, char *pszArgs[])
         }
     }
 
-    if (ShbCreateBlock(SHB_CTRLSvr, sizeof(CTRLConfig)) < 0)
+    if ((hShbCTRL = ShbCreateBlock(sizeof(CTRLConfig))) == SHB_INVALID_HANDLE)
         return (ErrGetErrorCode());
 
-    if ((hShbCTRL = ShbConnectBlock(SHB_CTRLSvr)) == SHB_INVALID_HANDLE)
-    {
-        ErrorPush();
-        ShbDestroyBlock(SHB_CTRLSvr);
-        return (ErrorPop());
-    }
-
     CTRLConfig     *pCTRLCfg = (CTRLConfig *) ShbLock(hShbCTRL);
 
     if (pCTRLCfg == NULL)
     {
         ErrorPush();
         ShbCloseBlock(hShbCTRL);
-        ShbDestroyBlock(SHB_CTRLSvr);
         return (ErrorPop());
     }
 
@@ -274,7 +280,6 @@ static int      SvrSetupCTRL(int iArgCount, char *pszArgs[])
     if ((hCTRLThread = SysCreateThread(CTRLThreadProc, NULL)) == SYS_INVALID_THREAD)
     {
         ShbCloseBlock(hShbCTRL);
-        ShbDestroyBlock(SHB_CTRLSvr);
         return (ErrGetErrorCode());
     }
 
@@ -309,7 +314,6 @@ static void     SvrCleanupCTRL(void)
 
 
     ShbCloseBlock(hShbCTRL);
-    ShbDestroyBlock(SHB_CTRLSvr);
 
 }
 
@@ -348,22 +352,14 @@ static int      SvrSetupFING(int iArgCount, char *pszArgs[])
         }
     }
 
-    if (ShbCreateBlock(SHB_FINGSvr, sizeof(FINGConfig)) < 0)
+    if ((hShbFING = ShbCreateBlock(sizeof(FINGConfig))) == SHB_INVALID_HANDLE)
         return (ErrGetErrorCode());
 
-    if ((hShbFING = ShbConnectBlock(SHB_FINGSvr)) == SHB_INVALID_HANDLE)
-    {
-        ErrorPush();
-        ShbDestroyBlock(SHB_FINGSvr);
-        return (ErrorPop());
-    }
-
     FINGConfig     *pFINGCfg = (FINGConfig *) ShbLock(hShbFING);
 
     if (pFINGCfg == NULL)
     {
         ShbCloseBlock(hShbFING);
-        ShbDestroyBlock(SHB_FINGSvr);
         return (ErrGetErrorCode());
     }
 
@@ -383,7 +379,6 @@ static int      SvrSetupFING(int iArgCount, char *pszArgs[])
     if ((hFINGThread = SysCreateThread(FINGThreadProc, NULL)) == SYS_INVALID_THREAD)
     {
         ShbCloseBlock(hShbFING);
-        ShbDestroyBlock(SHB_FINGSvr);
         return (ErrGetErrorCode());
     }
 
@@ -418,7 +413,6 @@ static void     SvrCleanupFING(void)
 
 
     ShbCloseBlock(hShbFING);
-    ShbDestroyBlock(SHB_FINGSvr);
 
 }
 
@@ -479,22 +473,14 @@ static int      SvrSetupPOP3(int iArgCount, char *pszArgs[])
         }
     }
 
-    if (ShbCreateBlock(SHB_POP3Svr, sizeof(POP3Config)) < 0)
+    if ((hShbPOP3 = ShbCreateBlock(sizeof(POP3Config))) == SHB_INVALID_HANDLE)
         return (ErrGetErrorCode());
 
-    if ((hShbPOP3 = ShbConnectBlock(SHB_POP3Svr)) == SHB_INVALID_HANDLE)
-    {
-        ErrorPush();
-        ShbDestroyBlock(SHB_POP3Svr);
-        return (ErrorPop());
-    }
-
     POP3Config     *pPOP3Cfg = (POP3Config *) ShbLock(hShbPOP3);
 
     if (pPOP3Cfg == NULL)
     {
         ShbCloseBlock(hShbPOP3);
-        ShbDestroyBlock(SHB_POP3Svr);
         return (ErrGetErrorCode());
     }
 
@@ -522,7 +508,6 @@ static int      SvrSetupPOP3(int iArgCount, char *pszArgs[])
     if ((hPOP3Thread = SysCreateThread(POP3ThreadProc, NULL)) == SYS_INVALID_THREAD)
     {
         ShbCloseBlock(hShbPOP3);
-        ShbDestroyBlock(SHB_POP3Svr);
         return (ErrGetErrorCode());
     }
 
@@ -556,7 +541,6 @@ static void     SvrCleanupPOP3(void)
 
 
     ShbCloseBlock(hShbPOP3);
-    ShbDestroyBlock(SHB_POP3Svr);
 
 }
 
@@ -569,6 +553,7 @@ static int      SvrSetupSMTP(int iArgCount, char *pszArgs[])
                     iSessionTimeout = STD_SERVER_SESSION_TIMEOUT,
                     iMaxRcpts = STD_SMTP_MAX_RCPTS,
                     iNumAddr = 0;
+    unsigned int    uPopAuthExpireTime = STD_POP3AUTH_EXPIRE_TIME;
     long            lMaxThreads = MAX_SMTP_THREADS;
     unsigned long   ulFlags = 0;
     ServerNetPath   SvrPath[MAX_SMTP_ACCEPT_ADDRESSES];
@@ -609,25 +594,23 @@ static int      SvrSetupSMTP(int iArgCount, char *pszArgs[])
                 if (++ii < iArgCount)
                     iMaxRcpts = atoi(pszArgs[ii]);
                 break;
+
+            case ('e'):
+                if (++ii < iArgCount)
+                    uPopAuthExpireTime = (unsigned int) atol(pszArgs[ii]);
+                break;
+
         }
     }
 
-    if (ShbCreateBlock(SHB_SMTPSvr, sizeof(SMTPConfig)) < 0)
+    if ((hShbSMTP = ShbCreateBlock(sizeof(SMTPConfig))) == SHB_INVALID_HANDLE)
         return (ErrGetErrorCode());
 
-    if ((hShbSMTP = ShbConnectBlock(SHB_SMTPSvr)) == SHB_INVALID_HANDLE)
-    {
-        ErrorPush();
-        ShbDestroyBlock(SHB_SMTPSvr);
-        return (ErrorPop());
-    }
-
     SMTPConfig     *pSMTPCfg = (SMTPConfig *) ShbLock(hShbSMTP);
 
     if (pSMTPCfg == NULL)
     {
         ShbCloseBlock(hShbSMTP);
-        ShbDestroyBlock(SHB_SMTPSvr);
         return (ErrGetErrorCode());
     }
 
@@ -638,6 +621,7 @@ static int      SvrSetupSMTP(int iArgCount, char *pszArgs[])
     pSMTPCfg->iSessionTimeout = iSessionTimeout;
     pSMTPCfg->iTimeout = STD_SERVER_TIMEOUT;
     pSMTPCfg->iMaxRcpts = iMaxRcpts;
+    pSMTPCfg->uPopAuthExpireTime = uPopAuthExpireTime;
     pSMTPCfg->iNumAddr = iNumAddr;
 
     for (int nn = 0; nn < iNumAddr; nn++)
@@ -649,7 +633,6 @@ static int      SvrSetupSMTP(int iArgCount, char *pszArgs[])
     if ((hSMTPThread = SysCreateThread(SMTPThreadProc, NULL)) == SYS_INVALID_THREAD)
     {
         ShbCloseBlock(hShbSMTP);
-        ShbDestroyBlock(SHB_SMTPSvr);
         return (ErrGetErrorCode());
     }
 
@@ -683,7 +666,6 @@ static void     SvrCleanupSMTP(void)
 
 
     ShbCloseBlock(hShbSMTP);
-    ShbDestroyBlock(SHB_SMTPSvr);
 
 }
 
@@ -711,7 +693,7 @@ static int      SvrSetupSMAIL(int iArgCount, char *pszArgs[])
                 if (++ii < iArgCount)
                     iNumSMAILThreads = atoi(pszArgs[ii]);
 
-                iNumSMAILThreads = min(MAX_SMAIL_THREADS, max(1, iNumSMAILThreads));
+                iNumSMAILThreads = Min(MAX_SMAIL_THREADS, Max(1, iNumSMAILThreads));
                 break;
 
             case ('t'):
@@ -735,33 +717,21 @@ static int      SvrSetupSMAIL(int iArgCount, char *pszArgs[])
         }
     }
 
-    if (ShbCreateBlock(SHB_SMAILSvr, sizeof(SMAILConfig)) < 0)
+    if ((hShbSMAIL = ShbCreateBlock(sizeof(SMAILConfig))) == SHB_INVALID_HANDLE)
         return (ErrGetErrorCode());
 
-    if ((hShbSMAIL = ShbConnectBlock(SHB_SMAILSvr)) == SHB_INVALID_HANDLE)
-    {
-        ErrorPush();
-        ShbDestroyBlock(SHB_SMAILSvr);
-        return (ErrorPop());
-    }
-
     SMAILConfig    *pSMAILCfg = (SMAILConfig *) ShbLock(hShbSMAIL);
 
     if (pSMAILCfg == NULL)
     {
         ErrorPush();
         ShbCloseBlock(hShbSMAIL);
-        ShbDestroyBlock(SHB_SMAILSvr);
 
         return (ErrorPop());
     }
 
     pSMAILCfg->ulFlags = ulFlags;
     pSMAILCfg->lThreadCount = 0;
-    pSMAILCfg->iTimeout = STD_SERVER_TIMEOUT;
-    pSMAILCfg->iRetryTimeout = iRetryTimeout;
-    pSMAILCfg->iRetryIncrRatio = iRetryIncrRatio;
-    pSMAILCfg->iMaxRetry = iMaxRetry;
 
 
     ShbUnlock(hShbSMAIL);
@@ -769,11 +739,15 @@ static int      SvrSetupSMAIL(int iArgCount, char *pszArgs[])
 ///////////////////////////////////////////////////////////////////////////////
 //  Initialize queue fs
 ///////////////////////////////////////////////////////////////////////////////
-    if (QueCreateQueue(SHB_SMAILQueue, NULL, iQueueSplitLevel) < 0)
+    char            szSpoolDir[SYS_MAX_PATH] = "";
+
+    SvrGetSpoolDir(szSpoolDir, sizeof(szSpoolDir));
+
+    if ((hSpoolQueue = QueOpen(szSpoolDir, iMaxRetry, iRetryTimeout, iRetryIncrRatio,
+                            iQueueSplitLevel)) == INVALID_QUEUE_HANDLE)
     {
         ErrorPush();
         ShbCloseBlock(hShbSMAIL);
-        ShbDestroyBlock(SHB_SMAILSvr);
 
         return (ErrorPop());
     }
@@ -818,12 +792,11 @@ static void     SvrCleanupSMAIL(void)
 
 
     ShbCloseBlock(hShbSMAIL);
-    ShbDestroyBlock(SHB_SMAILSvr);
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Close the mail queue
 ///////////////////////////////////////////////////////////////////////////////
-    QueCloseQueue(SHB_SMAILQueue);
+    QueClose(hSpoolQueue);
 
 }
 
@@ -851,32 +824,19 @@ static int      SvrSetupPSYNC(int iArgCount, char *pszArgs[])
                 if (++ii < iArgCount)
                     iNumSyncThreads = atoi(pszArgs[ii]);
 
-                iNumSyncThreads = min(MAX_PSYNC_NUM_THREADS, max(1, iNumSyncThreads));
+                iNumSyncThreads = Min(MAX_PSYNC_NUM_THREADS, Max(1, iNumSyncThreads));
                 break;
         }
     }
 
-    SemPSYNCThreads = SysCreateIPCName();
-
-    if ((SemSyncID = SysCreateSemaphore(iNumSyncThreads,
-                            SYS_DEFAULT_MAXCOUNT, SemPSYNCThreads)) == SYS_INVALID_SEMAPHORE)
+    if ((hSyncSem = SysCreateSemaphore(iNumSyncThreads, SYS_DEFAULT_MAXCOUNT)) == SYS_INVALID_SEMAPHORE)
         return (ErrGetErrorCode());
 
 
-    if (ShbCreateBlock(SHB_PSYNCSvr, sizeof(PSYNCConfig)) < 0)
+    if ((hShbPSYNC = ShbCreateBlock(sizeof(PSYNCConfig))) == SHB_INVALID_HANDLE)
     {
         ErrorPush();
-        SysCloseSemaphore(SemSyncID);
-        SysKillSemaphore(SemSyncID);
-        return (ErrorPop());
-    }
-
-    if ((hShbPSYNC = ShbConnectBlock(SHB_PSYNCSvr)) == SHB_INVALID_HANDLE)
-    {
-        ErrorPush();
-        ShbDestroyBlock(SHB_PSYNCSvr);
-        SysCloseSemaphore(SemSyncID);
-        SysKillSemaphore(SemSyncID);
+        SysCloseSemaphore(hSyncSem);
         return (ErrorPop());
     }
 
@@ -886,9 +846,7 @@ static int      SvrSetupPSYNC(int iArgCount, char *pszArgs[])
     {
         ErrorPush();
         ShbCloseBlock(hShbPSYNC);
-        ShbDestroyBlock(SHB_PSYNCSvr);
-        SysCloseSemaphore(SemSyncID);
-        SysKillSemaphore(SemSyncID);
+        SysCloseSemaphore(hSyncSem);
         return (ErrorPop());
     }
 
@@ -910,9 +868,7 @@ static int      SvrSetupPSYNC(int iArgCount, char *pszArgs[])
     if ((hPSYNCThread = SysCreateThread(PSYNCThreadProc, NULL)) == SYS_INVALID_THREAD)
     {
         ShbCloseBlock(hShbPSYNC);
-        ShbDestroyBlock(SHB_PSYNCSvr);
-        SysCloseSemaphore(SemSyncID);
-        SysKillSemaphore(SemSyncID);
+        SysCloseSemaphore(hSyncSem);
 
         return (ErrGetErrorCode());
     }
@@ -947,10 +903,108 @@ static void     SvrCleanupPSYNC(void)
 
 
     ShbCloseBlock(hShbPSYNC);
-    ShbDestroyBlock(SHB_PSYNCSvr);
 
-    SysCloseSemaphore(SemSyncID);
-    SysKillSemaphore(SemSyncID);
+    SysCloseSemaphore(hSyncSem);
+
+}
+
+
+
+static int      SvrSetupLMAIL(int iArgCount, char *pszArgs[])
+{
+
+    int             ii,
+                    iSleepTimeout = STD_LMAILTHREAD_SLEEP_TIME;
+    unsigned long   ulFlags = 0;
+
+    iNumLMAILThreads = STD_LMAIL_THREADS;
+
+    for (ii = 0; ii < iArgCount; ii++)
+    {
+        if ((pszArgs[ii][0] != '-') || (pszArgs[ii][1] != 'L'))
+            continue;
+
+        switch (pszArgs[ii][2])
+        {
+            case ('n'):
+                if (++ii < iArgCount)
+                    iNumLMAILThreads = atoi(pszArgs[ii]);
+
+                iNumLMAILThreads = Min(MAX_LMAIL_THREADS, Max(1, iNumLMAILThreads));
+                break;
+
+            case ('l'):
+                ulFlags |= LMAILF_LOG_ENABLED;
+                break;
+
+            case ('t'):
+                if (++ii < iArgCount)
+                    iSleepTimeout = atoi(pszArgs[ii]);
+                break;
+        }
+    }
+
+    if ((hShbLMAIL = ShbCreateBlock(sizeof(LMAILConfig))) == SHB_INVALID_HANDLE)
+        return (ErrGetErrorCode());
+
+    LMAILConfig    *pLMAILCfg = (LMAILConfig *) ShbLock(hShbLMAIL);
+
+    if (pLMAILCfg == NULL)
+    {
+        ErrorPush();
+        ShbCloseBlock(hShbLMAIL);
+
+        return (ErrorPop());
+    }
+
+    pLMAILCfg->ulFlags = ulFlags;
+    pLMAILCfg->iSleepTimeout = iSleepTimeout;
+    pLMAILCfg->lNumThreads = iNumLMAILThreads;
+    pLMAILCfg->lThreadCount = 0;
+
+
+    ShbUnlock(hShbLMAIL);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Create mailer threads
+///////////////////////////////////////////////////////////////////////////////
+    for (ii = 0; ii < iNumLMAILThreads; ii++)
+        hLMAILThreads[ii] = SysCreateThread(LMAILThreadProc, NULL);
+
+    return (0);
+
+}
+
+
+
+static void     SvrCleanupLMAIL(void)
+{
+
+///////////////////////////////////////////////////////////////////////////////
+//  Stop LMAIL Thread ( and unlock threads wait with "SysReleaseSemaphore" )
+///////////////////////////////////////////////////////////////////////////////
+    LMAILConfig    *pLMAILCfg = (LMAILConfig *) ShbLock(hShbLMAIL);
+
+    pLMAILCfg->ulFlags |= LMAILF_STOP_SERVER;
+
+    ShbUnlock(hShbLMAIL);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Wait LMAIL Threads
+///////////////////////////////////////////////////////////////////////////////
+    int             tt;
+
+    for (tt = 0; tt < iNumLMAILThreads; tt++)
+        SysWaitThread(hLMAILThreads[tt], SVR_EXIT_WAIT);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Close LMAIL Threads
+///////////////////////////////////////////////////////////////////////////////
+    for (tt = 0; tt < iNumLMAILThreads; tt++)
+        SysCloseThread(hLMAILThreads[tt], 1);
+
+
+    ShbCloseBlock(hShbLMAIL);
 
 }
 
@@ -965,14 +1019,18 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
 
     if (pszValue != NULL)
     {
-        strcpy(szMailPath, pszValue);
+        StrSNCpy(szMailPath, pszValue);
+        DelFinalSlash(szMailPath);
 
         SysFree(pszValue);
     }
 
+
     bServerDebug = false;
 
-    int             iMaxAccounts = MAX_XMAIL_ACCOUNTS;
+
+    int             iSndBufSize = -1,
+                    iRcvBufSize = -1;
 
     for (int ii = 0; ii < iArgCount; ii++)
     {
@@ -983,12 +1041,10 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
         {
             case ('s'):
                 if (++ii < iArgCount)
-                    strcpy(szMailPath, pszArgs[ii]);
-                break;
-
-            case ('k'):
-                if (++ii < iArgCount)
-                    uKeyMagic = (unsigned int) atol(pszArgs[ii]);
+                {
+                    StrSNCpy(szMailPath, pszArgs[ii]);
+                    DelFinalSlash(szMailPath);
+                }
                 break;
 
             case ('d'):
@@ -997,12 +1053,7 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
 
             case ('r'):
                 if (++ii < iArgCount)
-                    iLogRotateDays = atoi(pszArgs[ii]);
-                break;
-
-            case ('a'):
-                if (++ii < iArgCount)
-                    iMaxAccounts = atoi(pszArgs[ii]);
+                    iLogRotateHours = atoi(pszArgs[ii]);
                 break;
 
             case ('x'):
@@ -1014,10 +1065,27 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
                         ++iQueueSplitLevel;
                 }
                 break;
+
+            case ('R'):
+                if (++ii < iArgCount)
+                {
+                    iRcvBufSize = atoi(pszArgs[ii]);
+                    iRcvBufSize = NbrCeil(iRcvBufSize, 1024);
+                }
+                break;
+
+            case ('S'):
+                if (++ii < iArgCount)
+                {
+                    iSndBufSize = atoi(pszArgs[ii]);
+                    iSndBufSize = NbrCeil(iSndBufSize, 1024);
+                }
+                break;
+
         }
     }
 
-    if ((strlen(szMailPath) == 0) || !SysExistFile(szMailPath))
+    if ((strlen(szMailPath) == 0) || !SysExistDir(szMailPath))
     {
         ErrSetErrorCode(ERR_CONF_PATH);
         return (ERR_CONF_PATH);
@@ -1025,6 +1093,12 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
 
     AppendSlash(szMailPath);
 
+///////////////////////////////////////////////////////////////////////////////
+//  Setup library socket buffers
+///////////////////////////////////////////////////////////////////////////////
+    SysSetupSocketBuffers((iSndBufSize > 0) ? &iSndBufSize: NULL,
+            (iRcvBufSize > 0) ? &iRcvBufSize: NULL);
+
 ///////////////////////////////////////////////////////////////////////////////
 //  Setup shutdown file name ( must be called before any shutdown function )
 ///////////////////////////////////////////////////////////////////////////////
@@ -1033,7 +1107,7 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
 ///////////////////////////////////////////////////////////////////////////////
 //  Setup resource lockers
 ///////////////////////////////////////////////////////////////////////////////
-    if (RLckInitLockers(iMaxAccounts) < 0)
+    if (RLckInitLockers() < 0)
         return (ErrGetErrorCode());
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1058,7 +1132,8 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
     if ((UsrCheckUsersIndexes() < 0) ||
             (UsrCheckAliasesIndexes() < 0) ||
             (ExAlCheckAliasIndexes() < 0) ||
-            (MDomCheckDomainsIndexes() < 0))
+            (MDomCheckDomainsIndexes() < 0) ||
+            (ADomCheckDomainsIndexes() < 0))
     {
         ErrorPush();
         RLckCleanupLockers();
@@ -1077,16 +1152,6 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
         return (ErrorPop());
     }
 
-///////////////////////////////////////////////////////////////////////////////
-//  Initialize queue handler
-///////////////////////////////////////////////////////////////////////////////
-    if (QueHandlerInit() < 0)
-    {
-        ErrorPush();
-        RLckCleanupLockers();
-
-        return (ErrorPop());
-    }
 
     return (0);
 
@@ -1096,11 +1161,6 @@ static int      SvrSetup(int iArgCount, char *pszArgs[])
 
 static void     SvrCleanup(void)
 {
-///////////////////////////////////////////////////////////////////////////////
-//  Cleanup queue handler
-///////////////////////////////////////////////////////////////////////////////
-    QueHandlerCleanup();
-
 ///////////////////////////////////////////////////////////////////////////////
 //  Cleanup resource lockers
 ///////////////////////////////////////////////////////////////////////////////
@@ -1214,6 +1274,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
         ErrorPush();
         SysLogMessage(LOG_LEV_ERROR, "%s\n", ErrGetErrorString());
         StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
         SysCleanupLibrary();
         return (ErrorPop());
     }
@@ -1224,6 +1285,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
         SysLogMessage(LOG_LEV_ERROR, "%s\n", ErrGetErrorString());
         SvrCleanupSMAIL();
         StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
         SysCleanupLibrary();
         return (ErrorPop());
     }
@@ -1235,6 +1297,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
         SvrCleanupCTRL();
         SvrCleanupSMAIL();
         StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
         SysCleanupLibrary();
         return (ErrorPop());
     }
@@ -1247,6 +1310,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
         SvrCleanupCTRL();
         SvrCleanupSMAIL();
         StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
         SysCleanupLibrary();
         return (ErrorPop());
     }
@@ -1260,6 +1324,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
         SvrCleanupCTRL();
         SvrCleanupSMAIL();
         StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
         SysCleanupLibrary();
         return (ErrorPop());
     }
@@ -1274,6 +1339,23 @@ int             SvrMain(int iArgCount, char *pszArgs[])
         SvrCleanupCTRL();
         SvrCleanupSMAIL();
         StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
+        SysCleanupLibrary();
+        return (ErrorPop());
+    }
+
+    if (SvrSetupLMAIL(iMergeArgsCount, ppszMergeArgs) < 0)
+    {
+        ErrorPush();
+        SysLogMessage(LOG_LEV_ERROR, "%s\n", ErrGetErrorString());
+        SvrCleanupFING();
+        SvrCleanupPSYNC();
+        SvrCleanupSMTP();
+        SvrCleanupPOP3();
+        SvrCleanupCTRL();
+        SvrCleanupSMAIL();
+        StrFreeStrings(ppszMergeArgs);
+        SvrCleanup();
         SysCleanupLibrary();
         return (ErrorPop());
     }
@@ -1288,7 +1370,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
 ///////////////////////////////////////////////////////////////////////////////
 //  Server main loop
 ///////////////////////////////////////////////////////////////////////////////
-    for (; !SvrInShutdown();)
+    for (; !SvrInShutdown(true);)
     {
         SysSleep(SERVER_SLEEP_TIMESLICE);
 
@@ -1298,6 +1380,7 @@ int             SvrMain(int iArgCount, char *pszArgs[])
 ///////////////////////////////////////////////////////////////////////////////
 //  Goodbye cleanups
 ///////////////////////////////////////////////////////////////////////////////
+    SvrCleanupLMAIL();
     SvrCleanupFING();
     SvrCleanupPSYNC();
     SvrCleanupSMTP();
@@ -1331,7 +1414,7 @@ int             SvrStopServer(bool bWait)
     {
         int             iWaitTime = 0;
 
-        for (; SvrInShutdown(); iWaitTime += SERVER_SLEEP_TIMESLICE)
+        for (; SvrInShutdown(true); iWaitTime += SERVER_SLEEP_TIMESLICE)
             SysSleep(SERVER_SLEEP_TIMESLICE);
     }
 
@@ -1342,30 +1425,29 @@ int             SvrStopServer(bool bWait)
 
 
 
-bool            SvrInShutdown(void)
+bool            SvrInShutdown(bool bForceCheck)
 {
 
-    return ((SysExistFile(szShutdownFile)) ? true : false);
+    time_t          tNow = time(NULL);
+    static time_t   tLastCheck = 0;
+    static bool     bShutdown = false;
 
-}
-
-
-
-
-int             SvrShutdownCB(void *pData)
-{
-
-    time_t          tCurr = time(NULL);
-    static THRDLS time_t tLastCheck = 0;
-
-    if ((tCurr - tLastCheck) > SHUTDOWN_CHECK_TIME)
+    if (bForceCheck || (tNow > (tLastCheck + SHUTDOWN_CHECK_TIME)))
     {
-        tLastCheck = tCurr;
+        tLastCheck = tNow;
 
-        if (SvrInShutdown())
-            return (-1);
+        if (bServerShutdown)
+            bShutdown = true;
+        else if (SysExistFile(szShutdownFile))
+        {
+            bServerShutdown = true;
+
+            SysShutdownLibrary();
+
+            bShutdown = true;
+        }
     }
 
-    return (0);
+    return (bShutdown);
 
 }
