@@ -1,6 +1,6 @@
 /*
- *  MailSvr by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999  Davide Libenzi
+ *  XMail by Davide Libenzi ( Intranet and Internet mail server )
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Davide Libenzi <davide_libenzi@mycio.com>
+ *  Davide Libenzi <davidel@xmailserver.org>
  *
  */
 
@@ -71,7 +71,8 @@ int             MscGetTimeNbrString(char *pszTimeStr, int iStringSize, time_t tT
 int             MscGetTime(struct tm & tmLocal, int &iDiffHours, int &iDiffMins,
                         time_t tCurr = 0);
 int             MscGetTimeStr(char *pszTimeStr, int iStringSize, time_t tCurr = 0);
-int             MscGetDirectorySize(char const * pszPath, bool bRecurse, unsigned long &ulDirSize);
+int             MscGetDirectorySize(char const * pszPath, bool bRecurse, unsigned long &ulDirSize,
+                        unsigned long &ulNumFiles, int (*pFNValidate) (char const *) = NULL);
 FSCAN_HANDLE    MscFirstFile(char const * pszPath, int iListDirs, char *pszFileName);
 int             MscNextFile(FSCAN_HANDLE hFileScan, char *pszFileName);
 void            MscCloseFindFile(FSCAN_HANDLE hFileScan);
@@ -124,6 +125,15 @@ int             MscSetupServerNetPath(ServerNetPath & SvrPath, char const * pszC
                         int iDefPortNo);
 int             MscSplitAddressPort(char const * pszConnSpec, char * pszAddress,
                         int & iPortNo, int iDefPortNo);
+SYS_UINT16      MscReadUint16(void const * pData);
+SYS_UINT32      MscReadUint32(void const * pData);
+SYS_UINT64      MscReadUint64(void const * pData);
+void           *MscWriteUint16(void * pData, SYS_UINT16 uValue);
+void           *MscWriteUint32(void * pData, SYS_UINT32 uValue);
+void           *MscWriteUint64(void * pData, SYS_UINT64 uValue);
+int             MscCmdStringCheck(char const * pszString);
+int             MscGetSectionSize(FileSection const * pFS, unsigned long * pulSize);
+
 
 
 
