@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,15 +31,16 @@
 #include "BuffSock.h"
 #include "MD5.h"
 #include "MailConfig.h"
-#include "Queue.h"
 #include "UsrUtils.h"
 #include "SvrUtils.h"
+#include "MessQueue.h"
+#include "SMAILUtils.h"
+#include "QueueUtils.h"
 #include "MiscUtils.h"
 #include "Maildir.h"
 #include "POP3Svr.h"
 #include "POP3Utils.h"
 #include "SMTPUtils.h"
-#include "SMAILUtils.h"
 #include "MailSvr.h"
 
 
@@ -47,6 +48,7 @@
 
 
 #define UPOP_IPMAP_FILE         "pop3.ipmap.tab"
+#define POP3_IP_LOGFILE         ".ipconn"
 
 #define POPF_MSG_DELETED        (1 << 0)
 #define POPF_MSG_SENT           (1 << 1)
@@ -70,8 +72,11 @@ struct POP3MsgData
 
 struct POP3SessionData
 {
+    SYS_INET_ADDR   PeerInfo;
     UserInfo       *pUI;
     HSLIST          hMessageList;
+    POP3MsgData   **ppMsgArray;
+    int             iMsgListed;
     int             iMsgCount;
     unsigned long   ulMBSize;
     int             iLastAccessed;
@@ -83,15 +88,15 @@ struct POP3SessionData
 
 
 
+static int      UPopMailFileNameFilter(char const * pszFileName);
 static int      UPopFillMessageList(char const * pszBasePath, char const * pszSubPath,
                         HSLIST & hMessageList, int &iMsgCount, unsigned long &ulMBSize);
 static int      UPopBuildMessageList(UserInfo * pUI, HSLIST & hMessageList,
                         int *piMsgCount = NULL, unsigned long *pulMBSize = NULL);
 static void     UPopFreeMsgData(POP3MsgData * pPOPMD);
 static void     UPopFreeMessageList(HSLIST & hMessageList);
-static int      UPopCountMessages(HSLIST & hMessageList);
 static unsigned long UPopMessagesSize(HSLIST & hMessageList);
-static POP3MsgData *UPopMessageFromIndex(HSLIST & hMessageList, int iMsgIndex);
+static POP3MsgData *UPopMessageFromIndex(POP3SessionData *pPOPSD, int iMsgIndex);
 static int      UPopCheckPeerIP(UserInfo * pUI, SYS_INET_ADDR const & PeerInfo);
 static int      UPopUpdateMailbox(POP3SessionData * pPOPSD);
 static int      UPopCheckResponse(const char *pszResponse, char *pszMessage = NULL);
@@ -115,6 +120,7 @@ static int      UPopGetMailboxStatus(BSOCK_HANDLE hBSock, int &iMsgCount,
                         unsigned long &ulMailboxSize);
 static int      UPopRetrieveMessage(BSOCK_HANDLE hBSock, int iMsgIndex, const char *pszFileName);
 static int      UPopDeleteMessage(BSOCK_HANDLE hBSock, int iMsgIndex);
+static int      UPopGetIpLogFilePath(UserInfo * pUI, char *pszFilePath, int iMaxPath);
 
 
 
@@ -124,19 +130,43 @@ static int      UPopDeleteMessage(BSOCK_HANDLE hBSock, int iMsgIndex);
 
 
 
+static int      UPopMailFileNameFilter(char const * pszFileName)
+{
+
+    return ((*pszFileName != '.') ? 1: 0);
+
+}
 
 
-int             UPopGetMailboxSize(UserInfo * pUI, unsigned long &ulMBSize)
+
+int             UPopGetMailboxSize(UserInfo * pUI, unsigned long &ulMBSize,
+                        unsigned long &ulNumMessages)
 {
 
     char            szMBPath[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pUI, szMBPath, 0);
+    UsrGetMailboxPath(pUI, szMBPath, sizeof(szMBPath), 0);
 
 
-    if (MscGetDirectorySize(szMBPath, true, ulMBSize) < 0)
+    char            szResLock[SYS_MAX_PATH] = "";
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szMBPath, szResLock,
+                            sizeof(szResLock)));
+
+    if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
 
+    ulMBSize = 0;
+    ulNumMessages = 0;
+
+    if (MscGetDirectorySize(szMBPath, true, ulMBSize, ulNumMessages,
+            UPopMailFileNameFilter) < 0)
+    {
+        ErrorPush();
+        RLckUnlockSH(hResLock);
+        return (ErrorPop());
+    }
+
+    RLckUnlockSH(hResLock);
 
     return (0);
 
@@ -149,9 +179,10 @@ int             UPopGetMailboxSize(UserInfo * pUI, unsigned long &ulMBSize)
 int             UPopCheckMailboxSize(UserInfo * pUI, unsigned long *pulAvailSpace)
 {
 
-    unsigned long   ulMBSize = 0;
+    unsigned long   ulMBSize = 0,
+                    ulNumMessages = 0;
 
-    if (UPopGetMailboxSize(pUI, ulMBSize) < 0)
+    if (UPopGetMailboxSize(pUI, ulMBSize, ulNumMessages) < 0)
         return (ErrGetErrorCode());
 
 
@@ -161,7 +192,7 @@ int             UPopCheckMailboxSize(UserInfo * pUI, unsigned long *pulAvailSpac
     {
         unsigned long   ulMaxMBSize = (unsigned long) atol(pszMaxMBSize) * 1024;
 
-        if (ulMBSize > ulMaxMBSize)
+        if (ulMBSize >= ulMaxMBSize)
         {
             SysFree(pszMaxMBSize);
 
@@ -192,7 +223,7 @@ static int      UPopFillMessageList(char const * pszBasePath, char const * pszSu
     char            szScanPath[SYS_MAX_PATH] = "";
 
     if (pszSubPath == NULL)
-        strcpy(szScanPath, pszBasePath);
+        StrSNCpy(szScanPath, pszBasePath);
     else
         sprintf(szScanPath, "%s" SYS_SLASH_STR "%s", pszBasePath, pszSubPath);
 
@@ -225,7 +256,7 @@ static int      UPopFillMessageList(char const * pszBasePath, char const * pszSu
             ListLinkInit(pPOPMD);
 
             if (pszSubPath == NULL)
-                strcpy(pPOPMD->szMsgName, szFileName);
+                StrSNCpy(pPOPMD->szMsgName, szFileName);
             else
                 sprintf(pPOPMD->szMsgName, "%s" SYS_SLASH_STR "%s", pszSubPath, szFileName);
 
@@ -261,11 +292,12 @@ static int      UPopBuildMessageList(UserInfo * pUI, HSLIST & hMessageList,
 
     char            szMBPath[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pUI, szMBPath, 0);
+    UsrGetMailboxPath(pUI, szMBPath, sizeof(szMBPath), 0);
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -336,22 +368,6 @@ static void     UPopFreeMessageList(HSLIST & hMessageList)
 
 
 
-static int      UPopCountMessages(HSLIST & hMessageList)
-{
-
-    int             iMsgCount = 0;
-    POP3MsgData    *pPOPMD = (POP3MsgData *) ListFirst(hMessageList);
-
-    for (; pPOPMD != INVALID_SLIST_PTR; pPOPMD = (POP3MsgData *)
-            ListNext(hMessageList, (PLISTLINK) pPOPMD))
-        ++iMsgCount;
-
-    return (iMsgCount);
-
-}
-
-
-
 static unsigned long UPopMessagesSize(HSLIST & hMessageList)
 {
 
@@ -368,21 +384,11 @@ static unsigned long UPopMessagesSize(HSLIST & hMessageList)
 
 
 
-static POP3MsgData *UPopMessageFromIndex(HSLIST & hMessageList, int iMsgIndex)
+static POP3MsgData *UPopMessageFromIndex(POP3SessionData *pPOPSD, int iMsgIndex)
 {
 
-    POP3MsgData    *pPOPMD = (POP3MsgData *) ListFirst(hMessageList);
-
-    for (; pPOPMD != INVALID_SLIST_PTR; pPOPMD = (POP3MsgData *)
-            ListNext(hMessageList, (PLISTLINK) pPOPMD))
-    {
-        if (iMsgIndex == 0)
-            return (pPOPMD);
-
-        --iMsgIndex;
-    }
-
-    return (NULL);
+    return (((iMsgIndex >= 0) && (iMsgIndex < pPOPSD->iMsgListed)) ?
+            pPOPSD->ppMsgArray[iMsgIndex]: NULL);
 
 }
 
@@ -414,8 +420,8 @@ static int      UPopCheckPeerIP(UserInfo * pUI, SYS_INET_ADDR const & PeerInfo)
 
     char            szIPMapFile[SYS_MAX_PATH] = "";
 
-    UsrGetUserPath(pUI, szIPMapFile);
-    strcat(szIPMapFile, UPOP_IPMAP_FILE);
+    UsrGetUserPath(pUI, szIPMapFile, sizeof(szIPMapFile), 1);
+    StrNCat(szIPMapFile, UPOP_IPMAP_FILE, sizeof(szIPMapFile));
 
     if (SysExistFile(szIPMapFile) &&
             (MscCheckAllowedIP(szIPMapFile, PeerInfo, true) < 0))
@@ -437,6 +443,17 @@ POP3_HANDLE     UPopBuildSession(const char *pszDomain, const char *pszUsrName,
     if (pUI == NULL)
         return (INVALID_POP3_HANDLE);
 
+///////////////////////////////////////////////////////////////////////////////
+//  Check if the account is enabled for POP3 sessions
+///////////////////////////////////////////////////////////////////////////////
+    if (!UsrGetUserInfoVarInt(pUI, "PopEnable", 1))
+    {
+        UsrFreeUserInfo(pUI);
+
+        ErrSetErrorCode(ERR_USER_DISABLED);
+        return (INVALID_POP3_HANDLE);
+    }
+
 ///////////////////////////////////////////////////////////////////////////////
 //  Check if peer is allowed to connect from its IP
 ///////////////////////////////////////////////////////////////////////////////
@@ -469,6 +486,7 @@ POP3_HANDLE     UPopBuildSession(const char *pszDomain, const char *pszUsrName,
         return (INVALID_POP3_HANDLE);
     }
 
+    pPOPSD->PeerInfo = *pPeerInfo;
     pPOPSD->pUI = pUI;
     pPOPSD->iLastAccessed = 0;
     pPOPSD->iTimeout = STD_POP3_TIMEOUT;
@@ -482,6 +500,26 @@ POP3_HANDLE     UPopBuildSession(const char *pszDomain, const char *pszUsrName,
         return (INVALID_POP3_HANDLE);
     }
 
+    pPOPSD->iMsgListed = pPOPSD->iMsgCount;
+
+///////////////////////////////////////////////////////////////////////////////
+//  Build lookup array
+///////////////////////////////////////////////////////////////////////////////
+    if ((pPOPSD->ppMsgArray = (POP3MsgData   **) SysAlloc((pPOPSD->iMsgCount + 1) *
+                            sizeof(POP3MsgData *))) == NULL)
+    {
+        UPopFreeMessageList(pPOPSD->hMessageList);
+        SysFree(pPOPSD);
+        UsrPOP3Unlock(pUI);
+        UsrFreeUserInfo(pUI);
+        return (INVALID_POP3_HANDLE);
+    }
+
+    POP3MsgData    *pPOPMD = (POP3MsgData *) ListFirst(pPOPSD->hMessageList);
+
+    for (int ii = 0; (ii < pPOPSD->iMsgCount) && (pPOPMD != INVALID_SLIST_PTR);
+            pPOPMD = (POP3MsgData *) ListNext(pPOPSD->hMessageList, (PLISTLINK) pPOPMD), ii++)
+        pPOPSD->ppMsgArray[ii] = pPOPMD;
 
     return ((POP3_HANDLE) pPOPSD);
 
@@ -503,6 +541,8 @@ void            UPopReleaseSession(POP3_HANDLE hPOPSession, int iUpdate)
 
     UPopFreeMessageList(pPOPSD->hMessageList);
 
+    SysFree(pPOPSD->ppMsgArray);
+
     SysFree(pPOPSD);
 
 }
@@ -526,11 +566,12 @@ static int      UPopUpdateMailbox(POP3SessionData * pPOPSD)
 
     char            szMBPath[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pPOPSD->pUI, szMBPath, 0);
+    UsrGetMailboxPath(pPOPSD->pUI, szMBPath, sizeof(szMBPath), 0);
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -560,7 +601,7 @@ static int      UPopUpdateMailbox(POP3SessionData * pPOPSD)
 
 
 
-int             UPopGetSessionMsgCount(POP3_HANDLE hPOPSession)
+int             UPopGetSessionMsgCurrent(POP3_HANDLE hPOPSession)
 {
 
     POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
@@ -570,6 +611,18 @@ int             UPopGetSessionMsgCount(POP3_HANDLE hPOPSession)
 }
 
 
+
+int             UPopGetSessionMsgTotal(POP3_HANDLE hPOPSession)
+{
+
+    POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
+
+    return (pPOPSD->iMsgListed);
+
+}
+
+
+
 unsigned long   UPopGetSessionMBSize(POP3_HANDLE hPOPSession)
 {
 
@@ -600,7 +653,7 @@ int             UPopGetMessageSize(POP3_HANDLE hPOPSession, int iMsgIndex,
 
     ulMessageSize = 0;
 
-    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD->hMessageList, iMsgIndex - 1);
+    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD, iMsgIndex - 1);
 
     if (pPOPMD == NULL)
     {
@@ -628,7 +681,7 @@ int             UPopGetMessageUIDL(POP3_HANDLE hPOPSession, int iMsgIndex,
 
     POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
 
-    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD->hMessageList, iMsgIndex - 1);
+    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD, iMsgIndex - 1);
 
     if (pPOPMD == NULL)
     {
@@ -661,7 +714,7 @@ int             UPopDeleteMessage(POP3_HANDLE hPOPSession, int iMsgIndex)
 
     POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
 
-    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD->hMessageList, iMsgIndex - 1);
+    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD, iMsgIndex - 1);
 
     if (pPOPMD == NULL)
     {
@@ -747,7 +800,7 @@ int             UPopSessionSendMsg(POP3_HANDLE hPOPSession, int iMsgIndex,
 
     POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
 
-    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD->hMessageList, iMsgIndex - 1);
+    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD, iMsgIndex - 1);
 
     if (pPOPMD == NULL)
     {
@@ -767,9 +820,9 @@ int             UPopSessionSendMsg(POP3_HANDLE hPOPSession, int iMsgIndex,
 
     char            szMsgFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pPOPSD->pUI, szMsgFilePath);
+    UsrGetMailboxPath(pPOPSD->pUI, szMsgFilePath, sizeof(szMsgFilePath), 1);
 
-    strcat(szMsgFilePath, pPOPMD->szMsgName);
+    StrNCat(szMsgFilePath, pPOPMD->szMsgName, sizeof(szMsgFilePath));
 
 
     char            szResponse[256] = "";
@@ -779,10 +832,8 @@ int             UPopSessionSendMsg(POP3_HANDLE hPOPSession, int iMsgIndex,
     if (BSckSendString(hBSock, szResponse, pPOPSD->iTimeout) < 0)
         return (ErrGetErrorCode());
 
-    time_t          tCheckPoint = time(NULL);
-
-    if (SysSendFile(BSckGetAttachedSocket(hBSock), szMsgFilePath, pPOPSD->iTimeout,
-                    SvrShutdownCB, &tCheckPoint) < 0)
+    if (SysSendFile(BSckGetAttachedSocket(hBSock), szMsgFilePath, 0, (unsigned long) -1,
+                    pPOPSD->iTimeout) < 0)
         return (ErrGetErrorCode());
 
     if (BSckSendString(hBSock, ".", pPOPSD->iTimeout) < 0)
@@ -806,7 +857,7 @@ int             UPopSessionTopMsg(POP3_HANDLE hPOPSession, int iMsgIndex, int iN
 
     POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
 
-    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD->hMessageList, iMsgIndex - 1);
+    POP3MsgData    *pPOPMD = UPopMessageFromIndex(pPOPSD, iMsgIndex - 1);
 
     if (pPOPMD == NULL)
     {
@@ -826,9 +877,9 @@ int             UPopSessionTopMsg(POP3_HANDLE hPOPSession, int iMsgIndex, int iN
 
     char            szMsgFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pPOPSD->pUI, szMsgFilePath);
+    UsrGetMailboxPath(pPOPSD->pUI, szMsgFilePath, sizeof(szMsgFilePath), 1);
 
-    strcat(szMsgFilePath, pPOPMD->szMsgName);
+    StrNCat(szMsgFilePath, pPOPMD->szMsgName, sizeof(szMsgFilePath));
 
 
     FILE           *pMsgFile = fopen(szMsgFilePath, "rb");
@@ -855,18 +906,17 @@ int             UPopSessionTopMsg(POP3_HANDLE hPOPSession, int iMsgIndex, int iN
     bool            bSendingMsg = false;
     char            szMsgLine[2048] = "";
 
-    while ((iNumLines >= 0) &&
-            (MscGetString(pMsgFile, szMsgLine, sizeof(szMsgLine) - 1) != NULL))
+    while (MscGetString(pMsgFile, szMsgLine, sizeof(szMsgLine) - 1) != NULL)
     {
+        if (bSendingMsg && (--iNumLines < 0))
+            break;
+
         if (BSckSendString(hBSock, szMsgLine, pPOPSD->iTimeout) < 0)
         {
             fclose(pMsgFile);
             return (ErrGetErrorCode());
         }
 
-        if (bSendingMsg)
-            --iNumLines;
-
         if (!bSendingMsg && (strlen(szMsgLine) == 0))
             bSendingMsg = true;
 
@@ -896,6 +946,40 @@ int             UPopSessionTopMsg(POP3_HANDLE hPOPSession, int iMsgIndex, int iN
 
 
 
+int             UPopSaveUserIP(POP3_HANDLE hPOPSession)
+{
+
+    POP3SessionData *pPOPSD = (POP3SessionData *) hPOPSession;
+    char            szIpFilePath[SYS_MAX_PATH] = "";
+
+    UPopGetIpLogFilePath(pPOPSD->pUI, szIpFilePath, sizeof(szIpFilePath));
+
+
+    char            szIP[128] = "???.???.???.???";
+
+    SysInetNToA(pPOPSD->PeerInfo, szIP);
+
+
+    FILE           *pIpFile = fopen(szIpFilePath, "wt");
+
+    if (pIpFile == NULL)
+    {
+        ErrSetErrorCode(ERR_FILE_CREATE, szIpFilePath);
+        return (ERR_FILE_CREATE);
+    }
+
+
+    fprintf(pIpFile, "%s\n", szIP);
+
+
+    fclose(pIpFile);
+
+    return (0);
+
+}
+
+
+
 static int      UPopCheckResponse(const char *pszResponse, char *pszMessage)
 {
 
@@ -1045,12 +1129,11 @@ static BSOCK_HANDLE UPopCreateChannel(const char *pszServer, const char *pszUser
                         const char *pszPassword, bool bTryAuthAPOP)
 {
 
-    NET_ADDRESS     NetAddr;
+    ServerNetPath   NetPath;
 
-    if (MscGetServerAddress(pszServer, NetAddr) < 0)
+    if (MscSetupServerNetPath(NetPath, pszServer, STD_POP3_PORT) < 0)
         return (INVALID_BSOCK_HANDLE);
 
-
     SYS_SOCKET      SockFD = SysCreateSocket(AF_INET, SOCK_STREAM, 0);
 
     if (SockFD == SYS_INVALID_SOCKET)
@@ -1058,7 +1141,7 @@ static BSOCK_HANDLE UPopCreateChannel(const char *pszServer, const char *pszUser
 
     SYS_INET_ADDR   SvrAddr;
 
-    SysSetupAddress(SvrAddr, AF_INET, NetAddr, htons(STD_POP3_PORT));
+    SysSetupAddress(SvrAddr, AF_INET, NetPath.NetAddr, htons(NetPath.iPortNo));
 
     if (SysConnect(SockFD, &SvrAddr, sizeof(SvrAddr), STD_POP3_TIMEOUT) < 0)
     {
@@ -1091,7 +1174,7 @@ static BSOCK_HANDLE UPopCreateChannel(const char *pszServer, const char *pszUser
     char            szTimeStamp[256] = "";
 
     if (!bTryAuthAPOP ||
-            (MscExtractServerTimeStamp(szRTXBuffer, szTimeStamp, sizeof(szTimeStamp)) == NULL))
+            (MscExtractServerTimeStamp(szRTXBuffer, szTimeStamp, sizeof(szTimeStamp) - 1) == NULL))
     {
 ///////////////////////////////////////////////////////////////////////////////
 //  Try clear text authentication
@@ -1206,9 +1289,14 @@ static int      UPopRetrieveMessage(BSOCK_HANDLE hBSock, int iMsgIndex, const ch
         return (ErrGetErrorCode());
     }
 
+    int             iLineLength = 0,
+                    iGotNL,
+                    iGotNLPrev = 1;
+
     for (;;)
     {
-        if (BSckGetString(hBSock, szRTXBuffer, sizeof(szRTXBuffer) - 1, STD_POP3_TIMEOUT) == NULL)
+        if (BSckGetString(hBSock, szRTXBuffer, sizeof(szRTXBuffer) - 3,
+                STD_POP3_TIMEOUT, &iLineLength, &iGotNL) == NULL)
         {
             fclose(pMsgFile);
 
@@ -1216,10 +1304,28 @@ static int      UPopRetrieveMessage(BSOCK_HANDLE hBSock, int iMsgIndex, const ch
             return (ERR_POP3_RETR_BROKEN);
         }
 
-        if (strcmp(szRTXBuffer, ".") == 0)
+///////////////////////////////////////////////////////////////////////////////
+//  Check end of data condition
+///////////////////////////////////////////////////////////////////////////////
+        if (iGotNL && iGotNLPrev && (strcmp(szRTXBuffer, ".") == 0))
             break;
 
-        fprintf(pMsgFile, "%s\r\n", szRTXBuffer);
+///////////////////////////////////////////////////////////////////////////////
+//  Correctly terminate the line
+///////////////////////////////////////////////////////////////////////////////
+        if (iGotNL)
+            memcpy(szRTXBuffer + iLineLength, "\r\n", 3), iLineLength += 2;
+
+
+        if (!fwrite(szRTXBuffer, iLineLength, 1, pMsgFile))
+        {
+            fclose(pMsgFile);
+
+            ErrSetErrorCode(ERR_FILE_WRITE, pszFileName);
+            return (ERR_FILE_WRITE);
+        }
+
+        iGotNLPrev = iGotNL;
     }
 
     fclose(pMsgFile);
@@ -1247,7 +1353,8 @@ static int      UPopDeleteMessage(BSOCK_HANDLE hBSock, int iMsgIndex)
 
 
 int             UPopSyncRemoteLink(const char *pszSyncAddr, const char *pszRmtServer, const char *pszRmtName,
-                        const char *pszRmtPassword, const char *pszAuthType, const char * pszErrorAccount)
+                        const char *pszRmtPassword, const char *pszFetchHdrTags, const char *pszAuthType,
+                        const char *pszErrorAccount)
 {
 ///////////////////////////////////////////////////////////////////////////////
 //  Connection to POP3 server
@@ -1296,13 +1403,13 @@ int             UPopSyncRemoteLink(const char *pszSyncAddr, const char *pszRmtSe
 ///////////////////////////////////////////////////////////////////////////////
 //  Spool deliver fetched message
 ///////////////////////////////////////////////////////////////////////////////
-            if (USmlDeliverFetchedMsg(pszSyncAddr, szMsgFileName) < 0)
+            if (USmlDeliverFetchedMsg(pszSyncAddr, pszFetchHdrTags, szMsgFileName) < 0)
             {
 ///////////////////////////////////////////////////////////////////////////////
 //  If there's an error ( catch errors ) account try to deliver to this one
 ///////////////////////////////////////////////////////////////////////////////
                 if ((pszErrorAccount != NULL) &&
-                        (USmlDeliverFetchedMsg(pszErrorAccount, szMsgFileName) == 0))
+                        (USmlDeliverFetchedMsg(pszErrorAccount, NULL, szMsgFileName) == 0))
                 {
 ///////////////////////////////////////////////////////////////////////////////
 //  Delete remote message only if successfully delivered
@@ -1342,3 +1449,99 @@ int             UPopSyncRemoteLink(const char *pszSyncAddr, const char *pszRmtSe
     return (0);
 
 }
+
+
+
+static int      UPopGetIpLogFilePath(UserInfo * pUI, char *pszFilePath, int iMaxPath)
+{
+
+    UsrGetUserPath(pUI, pszFilePath, iMaxPath, 1);
+
+    StrNCat(pszFilePath, POP3_IP_LOGFILE, iMaxPath);
+
+    return (0);
+
+}
+
+
+
+int             UPopUserIpCheck(UserInfo * pUI, SYS_INET_ADDR const * pPeerInfo,
+                        unsigned int uExpireTime)
+{
+
+    char            szIpFilePath[SYS_MAX_PATH] = "";
+
+    UPopGetIpLogFilePath(pUI, szIpFilePath, sizeof(szIpFilePath));
+
+///////////////////////////////////////////////////////////////////////////////
+//  Load IP log file info and do expire check
+///////////////////////////////////////////////////////////////////////////////
+    SYS_FILE_INFO   FI;
+
+    if ((SysGetFileInfo(szIpFilePath, FI) < 0) ||
+            ((time_t) (FI.tMod + uExpireTime) < time(NULL)))
+    {
+        ErrSetErrorCode(ERR_NO_POP3_IP);
+        return (ERR_NO_POP3_IP);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Load IP from file
+///////////////////////////////////////////////////////////////////////////////
+    FILE           *pIpFile = fopen(szIpFilePath, "rt");
+
+    if (pIpFile == NULL)
+    {
+        ErrSetErrorCode(ERR_NO_POP3_IP);
+        return (ERR_NO_POP3_IP);
+    }
+
+    char            szIP[128] = "";
+
+    MscFGets(szIP, sizeof(szIP) - 1, pIpFile);
+
+    fclose(pIpFile);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Do IP matching
+///////////////////////////////////////////////////////////////////////////////
+    if (SysInetAddr(szIP) != SysGetAddrAddress(*pPeerInfo))
+    {
+        ErrSetErrorCode(ERR_NO_POP3_IP);
+        return (ERR_NO_POP3_IP);
+    }
+
+    return (0);
+
+}
+
+
+
+int             UPopGetLastLoginAddress(UserInfo * pUI, SYS_INET_ADDR * pAddress)
+{
+
+    char            szIpFilePath[SYS_MAX_PATH] = "";
+
+    UPopGetIpLogFilePath(pUI, szIpFilePath, sizeof(szIpFilePath));
+
+///////////////////////////////////////////////////////////////////////////////
+//  Load IP from file
+///////////////////////////////////////////////////////////////////////////////
+    FILE           *pIpFile = fopen(szIpFilePath, "rt");
+
+    if (pIpFile == NULL)
+    {
+        ErrSetErrorCode(ERR_NO_POP3_IP);
+        return (ERR_NO_POP3_IP);
+    }
+
+    char            szIP[128] = "";
+
+    MscFGets(szIP, sizeof(szIP) - 1, pIpFile);
+
+    fclose(pIpFile);
+
+
+    return (SysSetupAddress(*pAddress, AF_INET, SysInetAddr(szIP), 0));
+
+}
