@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -325,6 +325,54 @@ int             StrStringsCount(char const * const * ppszStrings)
 
 
 
+bool            StrStringsMatch(char const * const * ppszStrings, char const * pszMatch)
+{
+
+    int             ii;
+
+    for (ii = 0; ppszStrings[ii] != NULL; ii++)
+        if (strcmp(ppszStrings[ii], pszMatch) == 0)
+            return (true);
+
+
+    return (false);
+
+}
+
+
+
+bool            StrStringsIMatch(char const * const * ppszStrings, char const * pszMatch)
+{
+
+    int             ii;
+
+    for (ii = 0; ppszStrings[ii] != NULL; ii++)
+        if (stricmp(ppszStrings[ii], pszMatch) == 0)
+            return (true);
+
+
+    return (false);
+
+}
+
+
+
+bool            StrStringsRIWMatch(char const * const * pszMatches, char const * pszString)
+{
+
+    int             ii;
+
+    for (ii = 0; pszMatches[ii] != NULL; ii++)
+        if (StrIWildMatch(pszString, pszMatches[ii]))
+            return (true);
+
+
+    return (false);
+
+}
+
+
+
 char           *StrConcat(char const * const * ppszStrings, char const * pszCStr)
 {
 
@@ -439,22 +487,15 @@ char          **StrGetTabLineStrings(const char *pszUsrLine)
 int             StrWriteCRLFString(FILE * pFile, const char *pszString)
 {
 
-    int             iStrLength = strlen(pszString);
-    char           *pszBuffer = (char *) SysAlloc(iStrLength + 3);
+    unsigned int    uStrLength = strlen(pszString);
 
-    if (pszBuffer == NULL)
-        return (ErrGetErrorCode());
-
-    sprintf(pszBuffer, "%s\r\n", pszString);
-
-    if (fwrite(pszBuffer, iStrLength + 2, 1, pFile) == 0)
+    if ((uStrLength != 0) && (fwrite(pszString, uStrLength, 1, pFile) == 0))
     {
         ErrSetErrorCode(ERR_FILE_WRITE);
-        SysFree(pszBuffer);
         return (ERR_FILE_WRITE);
     }
 
-    SysFree(pszBuffer);
+    fputs("\r\n", pFile);
 
     return (0);
 
@@ -661,7 +702,7 @@ int             StrSplitString(char const * pszString, char const * pszSplitters
         {
             int             iUserLength = (int) (pszSplitChar - pszString);
 
-            StrNCpy(pszStrLeft, pszString, min(iUserLength + 1, iSizeLeft));
+            StrNCpy(pszStrLeft, pszString, Min(iUserLength + 1, iSizeLeft));
         }
 
         if (pszStrRight != NULL)
@@ -832,10 +873,12 @@ int             StrDynSize(DynString * pDS)
 
 
 
-int             StrDynAdd(DynString * pDS, char const * pszBuffer)
+int             StrDynAdd(DynString * pDS, char const * pszBuffer, int iStringSize)
 {
 
-    int	            iStringSize = strlen(pszBuffer);
+    if (iStringSize < 0)
+        iStringSize = strlen(pszBuffer);
+
 
     if ((pDS->iStringSize + iStringSize) >= pDS->iBufferSize)
     {
@@ -854,10 +897,12 @@ int             StrDynAdd(DynString * pDS, char const * pszBuffer)
         pDS->iBufferSize = iNewSize;
     }
 
-    strcpy(pDS->pszBuffer + pDS->iStringSize, pszBuffer);
+    memcpy(pDS->pszBuffer + pDS->iStringSize, pszBuffer, iStringSize);
 
     pDS->iStringSize += iStringSize;
 
+    pDS->pszBuffer[pDS->iStringSize] = '\0';
+
     return (0);
 
 }
