@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,6 +50,9 @@ char          **StrBuildList(char const *pszString, ...);
 char          **StrTokenize(const char *pszString, const char *pszTokenizer);
 void            StrFreeStrings(char **ppszStrings);
 int             StrStringsCount(char const * const * ppszStrings);
+bool            StrStringsMatch(char const * const * ppszStrings, char const * pszMatch);
+bool            StrStringsIMatch(char const * const * ppszStrings, char const * pszMatch);
+bool            StrStringsRIWMatch(char const * const * pszMatches, char const * pszString);
 char           *StrConcat(char const * const * ppszStrings, char const * pszCStr);
 char           *StrDeQuote(char *pszString, int iChar);
 char           *StrQuote(const char *pszString, int iChar);
@@ -72,7 +75,7 @@ int             StrDynFree(DynString * pDS);
 int             StrDynTruncate(DynString * pDS);
 char const     *StrDynGet(DynString * pDS);
 int             StrDynSize(DynString * pDS);
-int             StrDynAdd(DynString * pDS, char const * pszBuffer);
+int             StrDynAdd(DynString * pDS, char const * pszBuffer, int iStringSize = -1);
 
 
 
