@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,9 +29,9 @@
 ///////////////////////////////////////////////////////////////////////////////
 // Setup for the use of Maildir mailbox delivery
 ///////////////////////////////////////////////////////////////////////////////
-#if defined(__LINUX__) || defined(__SOLARIS__)
+#if defined(__LINUX__) || defined(__SOLARIS__) || defined(__FREEBSD__)
 #define CONFIG_MAILDIR
-#endif          // #if defined(__LINUX__) || defined(__SOLARIS__)
+#endif          // #if defined(__LINUX__) || defined(__SOLARIS__) || defined(__FREEBSD__)
 
 
 
