@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,10 +39,12 @@
 #define MAX_SPIN_COUNT              64
 #define SPIN_SLEEP_TIME             (50 * 1000)
 
-#define MIN_TCP_SEND_SIZE           1024
-#define MAX_TCP_SEND_SIZE           (1024 * 8)
-#define MIN_BYTES_SEC_TIMEOUT       64
-#define STD_SENDFILE_BLKSIZE        (4096 * 2)
+#define MIN_TCP_SEND_SIZE           (1024 * 8)
+#define MAX_TCP_SEND_SIZE           (1024 * 128)
+#define K_IO_TIME_RATIO             8
+
+#define MAX_STACK_SHIFT             2048
+#define STACK_ALIGN_BYTES           8
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Uncomment this if You want to use sendfile()
@@ -121,9 +123,10 @@ static void     SysSigChildHandler(int iSignal);
 static int      SysThreadSetup(ThrData * pTD);
 static void     SysThreadCleanup(ThrData * pTD);
 static int      SysExitPID(pid_t PID, int iExitCode);
-static int      SysWaitPID(pid_t PID, int * piExitCode, int iTimeout);
+static int      SysWaitPID(pid_t PID, int *piExitCode, int iTimeout);
 static void     SysBreakHandlerRoutine(int iSignal);
 static SYS_SPINLOCK SysTestAndSet(SYS_SPINLOCK * pSpinLock);
+static unsigned int SysStkCall(unsigned int (*pProc)(void *), void * pData);
 
 
 
@@ -133,10 +136,12 @@ static SYS_SPINLOCK SysTestAndSet(SYS_SPINLOCK * pSpinLock);
 
 
 
+static unsigned int uSRandBase;
 static pthread_mutex_t LogMutex = PTHREAD_MUTEX_INITIALIZER;
 static void     (*SysBreakHandler) (void) = NULL;
-static SYS_SPINLOCK WaitPIDSpin = 0;
-static SYS_LIST_HEAD(WaitPIDList);
+static SYS_SPINLOCK WaitPIDSpin = SYS_SPINLOCK_UNLOCKED;
+static          SYS_LIST_HEAD(WaitPIDList);
+static int      iSndBufSize = -1, iRcvBufSize = -1;
 
 
 
@@ -173,6 +178,8 @@ static void     SysIgnoreProc(int iSignal)
 int             SysInitLibrary(void)
 {
 
+    tzset();
+    uSRandBase = (unsigned int) time(NULL);
 
     if (SysThreadSetup(NULL) < 0)
         return (ErrGetErrorCode());
@@ -194,6 +201,32 @@ void            SysCleanupLibrary(void)
 
 
 
+int             SysShutdownLibrary(int iMode)
+{
+
+    kill(0, SIGQUIT);
+
+    return (0);
+
+}
+
+
+
+int             SysSetupSocketBuffers(int *piSndBufSize, int *piRcvBufSize)
+{
+
+    if (piSndBufSize != NULL)
+        iSndBufSize = *piSndBufSize;
+
+    if (piRcvBufSize != NULL)
+        iRcvBufSize = *piRcvBufSize;
+
+    return (0);
+
+}
+
+
+
 SYS_SOCKET      SysCreateSocket(int iAddressFamily, int iType, int iProtocol)
 {
 
@@ -248,6 +281,23 @@ static int      SysSetSockNoDelay(SYS_SOCKET SockFD, int iNoDelay)
 
 static int      SysSetSocketsOptions(SYS_SOCKET SockFD)
 {
+///////////////////////////////////////////////////////////////////////////////
+//  Set socket buffer sizes
+///////////////////////////////////////////////////////////////////////////////
+    if (iSndBufSize > 0)
+    {
+        int             iSize = iSndBufSize;
+
+        setsockopt((int) SockFD, SOL_SOCKET, SO_SNDBUF, (const char *) &iSize, sizeof(iSize));
+    }
+
+    if (iRcvBufSize > 0)
+    {
+        int             iSize = iRcvBufSize;
+
+        setsockopt((int) SockFD, SOL_SOCKET, SO_RCVBUF, (const char *) &iSize, sizeof(iSize));
+    }
+
 
     int             iActivate = 1;
 
@@ -317,30 +367,32 @@ void            SysListenSocket(SYS_SOCKET SockFD, int iConnections)
 int             SysRecvData(SYS_SOCKET SockFD, char *pszBuffer, int iBufferSize, int iTimeout)
 {
 
-    fd_set          rfds;
-    struct timeval  tv;
+    struct pollfd   pfds;
 
-    ZeroData(tv);
-    tv.tv_sec = iTimeout;
-    tv.tv_usec = 0;
+    ZeroData(pfds);
+    pfds.fd = (int) SockFD;
+    pfds.events = POLLIN;
 
-    FD_ZERO(&rfds);
-    FD_SET((int) SockFD, &rfds);
 
-    if (select((int) SockFD + 1, &rfds, (fd_set *) 0, (fd_set *) 0, &tv) == -1)
+    int             iPollResult = poll(&pfds, 1, iTimeout * 1000);
+
+
+    if (iPollResult == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
         return (ERR_NETWORK);
     }
 
-    if (!FD_ISSET((int) SockFD, &rfds))
+    if (iPollResult == 0)
     {
         ErrSetErrorCode(ERR_TIMEOUT);
         return (ERR_TIMEOUT);
     }
 
+
     int             iRecvBytes = recv((int) SockFD, pszBuffer, iBufferSize, 0);
 
+
     if (iRecvBytes == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
@@ -379,31 +431,33 @@ int             SysRecvDataFrom(SYS_SOCKET SockFD, struct sockaddr * pFrom, int
                         char *pszBuffer, int iBufferSize, int iTimeout)
 {
 
-    fd_set          rfds;
-    struct timeval  tv;
+    struct pollfd   pfds;
 
-    ZeroData(tv);
-    tv.tv_sec = iTimeout;
-    tv.tv_usec = 0;
+    ZeroData(pfds);
+    pfds.fd = (int) SockFD;
+    pfds.events = POLLIN;
 
-    FD_ZERO(&rfds);
-    FD_SET((int) SockFD, &rfds);
 
-    if (select((int) SockFD + 1, &rfds, (fd_set *) 0, (fd_set *) 0, &tv) == -1)
+    int             iPollResult = poll(&pfds, 1, iTimeout * 1000);
+
+
+    if (iPollResult == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
         return (ERR_NETWORK);
     }
 
-    if (!FD_ISSET((int) SockFD, &rfds))
+    if (iPollResult == 0)
     {
         ErrSetErrorCode(ERR_TIMEOUT);
         return (ERR_TIMEOUT);
     }
 
+
     socklen_t       SockALen = (socklen_t) iFromlen;
     int             iRecvBytes = recvfrom((int) SockFD, pszBuffer, iBufferSize, 0, pFrom, &SockALen);
 
+
     if (iRecvBytes == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
@@ -419,30 +473,32 @@ int             SysRecvDataFrom(SYS_SOCKET SockFD, struct sockaddr * pFrom, int
 int             SysSendData(SYS_SOCKET SockFD, char const * pszBuffer, int iBufferSize, int iTimeout)
 {
 
-    fd_set          wfds;
-    struct timeval  tv;
+    struct pollfd   pfds;
 
-    ZeroData(tv);
-    tv.tv_sec = iTimeout;
-    tv.tv_usec = 0;
+    ZeroData(pfds);
+    pfds.fd = (int) SockFD;
+    pfds.events = POLLOUT;
 
-    FD_ZERO(&wfds);
-    FD_SET((int) SockFD, &wfds);
 
-    if (select((int) SockFD + 1, (fd_set *) 0, &wfds, (fd_set *) 0, &tv) == -1)
+    int             iPollResult = poll(&pfds, 1, iTimeout * 1000);
+
+
+    if (iPollResult == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
         return (ERR_NETWORK);
     }
 
-    if (!FD_ISSET((int) SockFD, &wfds))
+    if (iPollResult == 0)
     {
         ErrSetErrorCode(ERR_TIMEOUT);
         return (ERR_TIMEOUT);
     }
 
+
     int             iSendBytes = send((int) SockFD, pszBuffer, iBufferSize, 0);
 
+
     if (iSendBytes == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
@@ -481,30 +537,32 @@ int             SysSendDataTo(SYS_SOCKET SockFD, const struct sockaddr * pTo,
                         int iToLen, char const * pszBuffer, int iBufferSize, int iTimeout)
 {
 
-    fd_set          wfds;
-    struct timeval  tv;
+    struct pollfd   pfds;
 
-    ZeroData(tv);
-    tv.tv_sec = iTimeout;
-    tv.tv_usec = 0;
+    ZeroData(pfds);
+    pfds.fd = (int) SockFD;
+    pfds.events = POLLOUT;
 
-    FD_ZERO(&wfds);
-    FD_SET((int) SockFD, &wfds);
 
-    if (select((int) SockFD + 1, (fd_set *) 0, &wfds, (fd_set *) 0, &tv) == -1)
+    int             iPollResult = poll(&pfds, 1, iTimeout * 1000);
+
+
+    if (iPollResult == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
         return (ERR_NETWORK);
     }
 
-    if (!FD_ISSET((int) SockFD, &wfds))
+    if (iPollResult == 0)
     {
         ErrSetErrorCode(ERR_TIMEOUT);
         return (ERR_TIMEOUT);
     }
 
+
     int             iSendBytes = sendto((int) SockFD, pszBuffer, iBufferSize, 0, pTo, iToLen);
 
+
     if (iSendBytes == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
@@ -537,25 +595,25 @@ int             SysConnect(SYS_SOCKET SockFD, const SYS_INET_ADDR * pSockName, i
         return (ERR_NETWORK);
     }
 
-    fd_set          wfds;
-    struct timeval  tv;
+    struct pollfd   pfds;
 
-    FD_ZERO(&wfds);
-    FD_SET((int) SockFD, &wfds);
-    tv.tv_sec = iTimeout;
-    tv.tv_usec = 0;
+    ZeroData(pfds);
+    pfds.fd = (int) SockFD;
+    pfds.events = POLLOUT;
 
-    if (select((int) SockFD + 1, (fd_set *) 0, &wfds, (fd_set *) 0, &tv) == -1)
+
+    int             iPollResult = poll(&pfds, 1, iTimeout * 1000);
+
+
+    SysSetSockNoDelay(SockFD, 0);
+
+    if (iPollResult == -1)
     {
-        SysSetSockNoDelay(SockFD, 0);
-
         ErrSetErrorCode(ERR_NETWORK);
         return (ERR_NETWORK);
     }
 
-    SysSetSockNoDelay(SockFD, 0);
-
-    if (!FD_ISSET((int) SockFD, &wfds))
+    if (iPollResult == 0)
     {
         ErrSetErrorCode(ERR_TIMEOUT);
         return (ERR_TIMEOUT);
@@ -570,28 +628,29 @@ int             SysConnect(SYS_SOCKET SockFD, const SYS_INET_ADDR * pSockName, i
 SYS_SOCKET      SysAccept(SYS_SOCKET SockFD, SYS_INET_ADDR * pSockName, int *iNameLen, int iTimeout)
 {
 
-    fd_set          rfds;
-    struct timeval  tv;
+    struct pollfd   pfds;
 
-    ZeroData(tv);
-    tv.tv_sec = iTimeout;
-    tv.tv_usec = 0;
+    ZeroData(pfds);
+    pfds.fd = (int) SockFD;
+    pfds.events = POLLIN;
 
-    FD_ZERO(&rfds);
-    FD_SET((int) SockFD, &rfds);
 
-    if (select((int) SockFD + 1, &rfds, (fd_set *) 0, (fd_set *) 0, &tv) == -1)
+    int             iPollResult = poll(&pfds, 1, iTimeout * 1000);
+
+
+    if (iPollResult == -1)
     {
         ErrSetErrorCode(ERR_NETWORK);
         return (ERR_NETWORK);
     }
 
-    if (!FD_ISSET((int) SockFD, &rfds))
+    if (iPollResult == 0)
     {
         ErrSetErrorCode(ERR_TIMEOUT);
         return (ERR_TIMEOUT);
     }
 
+
     socklen_t       SockALen = (socklen_t) * iNameLen;
     int             iAcptSock = accept((int) SockFD,
             (struct sockaddr *) & pSockName->Addr, &SockALen);
@@ -650,15 +709,15 @@ int             SysSelect(int iMaxFD, SYS_fd_set * pReadFDs, SYS_fd_set * pWrite
 
 
 
-int             SysSendFile(SYS_SOCKET SockFD, char const * pszFileName, int iTimeout,
-                        int (*pSendCB) (void *), void *pUserData)
+int             SysSendFile(SYS_SOCKET SockFD, char const * pszFileName, unsigned long ulBaseOffset,
+                        unsigned long ulEndOffset, int iTimeout)
 {
 
     int             iFileID = open(pszFileName, O_RDONLY);
 
     if (iFileID == -1)
     {
-        ErrSetErrorCode(ERR_FILE_OPEN);
+        ErrSetErrorCode(ERR_FILE_OPEN, pszFileName);
         return (ERR_FILE_OPEN);
     }
 
@@ -667,36 +726,60 @@ int             SysSendFile(SYS_SOCKET SockFD, char const * pszFileName, int iTi
     lseek(iFileID, 0, SEEK_SET);
 
 #ifdef USE_SENDFILE
+///////////////////////////////////////////////////////////////////////////////
+//  Set send timeout
+///////////////////////////////////////////////////////////////////////////////
+    socklen_t       OptLenght = sizeof(struct timeval);
+    struct timeval  oldTV,
+                    newTV;
 
-    unsigned long   ulSent = 0;
-
-    while (ulSent < ulFileSize)
+    if (getsockopt((int) SockFD, SOL_SOCKET, SO_SNDTIMEO, &oldTV, &OptLenght))
     {
-        unsigned long   ulToSend = min(STD_SENDFILE_BLKSIZE, ulFileSize - ulSent);
-        off_t           ulStartOffset = (off_t) ulSent;
+        close(iFileID);
+        ErrSetErrorCode(ERR_GETSOCKOPT);
+        return (ERR_GETSOCKOPT);
+    }
+
+    newTV.tv_sec = iTimeout;
+    newTV.tv_usec = 0;
+    setsockopt((int) SockFD, SOL_SOCKET, SO_SNDTIMEO, &newTV, sizeof(newTV));
+
+///////////////////////////////////////////////////////////////////////////////
+//  Send the file
+///////////////////////////////////////////////////////////////////////////////
+    unsigned long   ulSndBuffSize = MIN_TCP_SEND_SIZE,
+                    ulCurrOffset = ulBaseOffset,
+                    ulSndEndOffset = (ulEndOffset != (unsigned long) -1) ? ulEndOffset: ulFileSize;
+    time_t          tStart;
+
+    while (ulCurrOffset < ulSndEndOffset)
+    {
+        unsigned long   ulToSend = Min(ulSndBuffSize, ulSndEndOffset - ulCurrOffset);
+        off_t           ulStartOffset = (off_t) ulCurrOffset;
 
 
+        tStart = time(NULL);
         unsigned long   ulSendSize = (unsigned long) sendfile((int) SockFD, iFileID,
                 &ulStartOffset, ulToSend);
 
 
         if (ulSendSize != ulToSend)
         {
+            setsockopt((int) SockFD, SOL_SOCKET, SO_SNDTIMEO, &oldTV, sizeof(oldTV));
             close(iFileID);
             ErrSetErrorCode(ERR_SENDFILE);
             return (ERR_SENDFILE);
         }
 
-        if ((pSendCB != NULL) && (pSendCB(pUserData) < 0))
-        {
-            close(iFileID);
-            ErrSetErrorCode(ERR_USER_BREAK);
-            return (ERR_USER_BREAK);
-        }
+        if ((((time(NULL) - tStart) * K_IO_TIME_RATIO) < iTimeout) &&
+                (ulSndBuffSize < MAX_TCP_SEND_SIZE))
+            ulSndBuffSize = Min(ulSndBuffSize * 2, MAX_TCP_SEND_SIZE);
 
-        ulSent += ulToSend;
+        ulCurrOffset += ulToSend;
     }
 
+    setsockopt((int) SockFD, SOL_SOCKET, SO_SNDTIMEO, &oldTV, sizeof(oldTV));
+
 #else           // #ifdef USE_SENDFILE
 
     void           *pMapAddress = (void *) mmap((char *) 0, (size_t) ulFileSize, PROT_READ,
@@ -709,29 +792,21 @@ int             SysSendFile(SYS_SOCKET SockFD, char const * pszFileName, int iTi
         return (ERR_MMAP);
     }
 
-///////////////////////////////////////////////////////////////////////////////
-//  Setup the maximum data transfer size
-///////////////////////////////////////////////////////////////////////////////
-    int             iSndBuffSize = 0;
-    socklen_t       OptLenght = sizeof(iSndBuffSize);
-
-    if (getsockopt(SockFD, SOL_SOCKET, SO_SNDBUF, (char *) &iSndBuffSize, &OptLenght) != 0)
-        iSndBuffSize = MIN_TCP_SEND_SIZE;
-    else
-        iSndBuffSize = min(iSndBuffSize, MAX_TCP_SEND_SIZE);
-
 ///////////////////////////////////////////////////////////////////////////////
 //  Send the file
 ///////////////////////////////////////////////////////////////////////////////
-    unsigned long   ulSentBytes = 0;
-    char           *pszBuffer = (char *) pMapAddress;
+    int             iSndBuffSize = MIN_TCP_SEND_SIZE;
+    unsigned long   ulCurrOffset = ulBaseOffset,
+                    ulSndEndOffset = (ulEndOffset != (unsigned long) -1) ? ulEndOffset: ulFileSize;
+    char           *pszBuffer = (char *) pMapAddress + ulBaseOffset;
+    time_t          tStart;
 
-    while (ulSentBytes < ulFileSize)
+    while (ulCurrOffset < ulSndEndOffset)
     {
-        int             iCurrSend = (int) min(iSndBuffSize, ulFileSize - ulSentBytes);
+        int             iCurrSend = (int) Min(iSndBuffSize, ulSndEndOffset - ulCurrOffset);
 
-        if ((iCurrSend = SysSendData(SockFD, pszBuffer, iCurrSend,
-                                max(iTimeout, iCurrSend / MIN_BYTES_SEC_TIMEOUT))) < 0)
+        tStart = time(NULL);
+        if ((iCurrSend = SysSendData(SockFD, pszBuffer, iCurrSend, iTimeout)) < 0)
         {
             ErrorPush();
             munmap((char *) pMapAddress, (size_t) ulFileSize);
@@ -739,16 +814,12 @@ int             SysSendFile(SYS_SOCKET SockFD, char const * pszFileName, int iTi
             return (ErrorPop());
         }
 
-        if ((pSendCB != NULL) && (pSendCB(pUserData) < 0))
-        {
-            munmap((char *) pMapAddress, (size_t) ulFileSize);
-            close(iFileID);
-            ErrSetErrorCode(ERR_USER_BREAK);
-            return (ERR_USER_BREAK);
-        }
+        if ((((time(NULL) - tStart) * K_IO_TIME_RATIO) < iTimeout) &&
+                (iSndBuffSize < MAX_TCP_SEND_SIZE))
+            iSndBuffSize = Min(iSndBuffSize * 2, MAX_TCP_SEND_SIZE);
 
         pszBuffer += iCurrSend;
-        ulSentBytes += (unsigned long) iCurrSend;
+        ulCurrOffset += (unsigned long) iCurrSend;
     }
 
     munmap((char *) pMapAddress, (size_t) ulFileSize);
@@ -823,7 +894,9 @@ int             SysGetHostByAddr(SYS_INET_ADDR const & AddrInfo, char *pszFQDN)
                             AF_INET, &HostEnt, szBuffer, sizeof(szBuffer), &pHostEnt, &iErrorNo) != 0) ||
             (pHostEnt == NULL) || (pHostEnt->h_name == NULL))
     {
-        ErrSetErrorCode(ERR_GET_SOCK_HOST, SysInetNToA(AddrInfo));
+        char            szIP[128] = "???.???.???.???";
+
+        ErrSetErrorCode(ERR_GET_SOCK_HOST, SysInetNToA(AddrInfo, szIP));
         return (ERR_GET_SOCK_HOST);
     }
 
@@ -876,10 +949,24 @@ int             SysGetSockInfo(SYS_SOCKET SockFD, SYS_INET_ADDR & AddrInfo)
 
 
 
-char const     *SysInetNToA(SYS_INET_ADDR const & AddrInfo)
+char           *SysInetNToA(SYS_INET_ADDR const & AddrInfo, char *pszIP)
 {
 
-    return (inet_ntoa(AddrInfo.Addr.sin_addr));
+    union
+    {
+        unsigned int    a;
+        unsigned char   b[4];
+    }               UAddr;
+
+    memcpy(&UAddr, &AddrInfo.Addr.sin_addr, sizeof(UAddr));
+
+    sprintf(pszIP, "%u.%u.%u.%u",
+            (unsigned int) UAddr.b[0],
+            (unsigned int) UAddr.b[1],
+            (unsigned int) UAddr.b[2],
+            (unsigned int) UAddr.b[3]);
+
+    return (pszIP);
 
 }
 
@@ -1020,7 +1107,12 @@ int             SysReleaseSemaphore(SYS_SEMAPHORE hSemaphore, int iCount)
     pSD->iSemCounter += iCount;
 
     if (pSD->iSemCounter > 0)
-        pthread_cond_broadcast(&pSD->WaitCond);
+    {
+        if (pSD->iSemCounter > 1)
+            pthread_cond_broadcast(&pSD->WaitCond);
+        else
+            pthread_cond_signal(&pSD->WaitCond);
+    }
 
     pthread_mutex_unlock(&pSD->Mtx);
 
@@ -1174,7 +1266,7 @@ int             SysUnlockMutex(SYS_MUTEX hMutex)
 
     pMD->iLocked = 0;
 
-    pthread_cond_broadcast(&pMD->WaitCond);
+    pthread_cond_signal(&pMD->WaitCond);
 
     pthread_mutex_unlock(&pMD->Mtx);
 
@@ -1330,7 +1422,10 @@ int             SysSetEvent(SYS_EVENT hEvent)
 
     pED->iSignaled = 1;
 
-    pthread_cond_broadcast(&pED->WaitCond);
+    if (pED->iManualReset)
+        pthread_cond_broadcast(&pED->WaitCond);
+    else
+        pthread_cond_signal(&pED->WaitCond);
 
     pthread_mutex_unlock(&pED->Mtx);
 
@@ -1409,7 +1504,7 @@ static void    *SysThreadStartup(void *pThreadData)
 
     pthread_cleanup_push((void (*) (void *)) SysThreadCleanup, pTD);
 
-    pTD->iExitCode = iExitCode = pTD->ThreadProc(pTD->pThreadData);
+    pTD->iExitCode = iExitCode = SysStkCall(pTD->ThreadProc, pTD->pThreadData);
 
     pthread_cleanup_pop(1);
 
@@ -1510,14 +1605,21 @@ static int      SysThreadSetup(ThrData * pTD)
     sigemptyset(&SigMask);
     sigaddset(&SigMask, SIGALRM);
     sigaddset(&SigMask, SIGINT);
-    sigaddset(&SigMask, SIGQUIT);
     sigaddset(&SigMask, SIGHUP);
+    sigaddset(&SigMask, SIGSTOP);
 
     pthread_sigmask(SIG_BLOCK, &SigMask, NULL);
 
 
-    signal(SIGPIPE, SysIgnoreProc);
+    sigemptyset(&SigMask);
+    sigaddset(&SigMask, SIGQUIT);
 
+    pthread_sigmask(SIG_UNBLOCK, &SigMask, NULL);
+
+
+    signal(SIGQUIT, SysIgnoreProc);
+
+    signal(SIGPIPE, SysIgnoreProc);
 
     signal(SIGCHLD, SysSigChildHandler);
 
@@ -1720,13 +1822,13 @@ static int      SysExitPID(pid_t PID, int iExitCode)
 
 
 
-static int      SysWaitPID(pid_t PID, int * piExitCode, int iTimeout)
+static int      SysWaitPID(pid_t PID, int *piExitCode, int iTimeout)
 {
 
     PIDWaitData     PWD;
 
     ZeroData(PWD);
-    SYS_INIT_LIST_HEAD(&PWD.LLink);
+    SYS_INIT_LIST_LINK(&PWD.LLink);
     PWD.WaitThreadId = pthread_self();
     PWD.PID = PID;
     PWD.iExitCode = -1;
@@ -1743,13 +1845,33 @@ static int      SysWaitPID(pid_t PID, int * piExitCode, int iTimeout)
 ///////////////////////////////////////////////////////////////////////////////
 //  Wait for PID exit
 ///////////////////////////////////////////////////////////////////////////////
-    iTimeout *= 1000;
+    int             iExitStatus = 0;
 
-    while ((iTimeout > 0) && (PWD.PID != 0))
+    if (waitpid((pid_t) PID, &iExitStatus, WUNTRACED | WNOHANG) != PID)
     {
-        SysMsSleep(WAIT_PID_TIME_STEP);
+        if (errno == ECHILD)
+        {
+            SysSpinAcquire(&WaitPIDSpin);
+            SYS_LIST_DEL(&PWD.LLink);
+            SysSpinRelease(&WaitPIDSpin);
 
-        iTimeout -= WAIT_PID_TIME_STEP;
+            ErrSetErrorCode(ERR_PROCESS_EXECUTE);
+            return (ERR_PROCESS_EXECUTE);
+        }
+
+        iTimeout *= 1000;
+
+        while ((iTimeout > 0) && (PWD.PID != 0))
+        {
+            SysMsSleep(WAIT_PID_TIME_STEP);
+
+            iTimeout -= WAIT_PID_TIME_STEP;
+        }
+    }
+    else
+    {
+        PWD.PID = 0;
+        PWD.iExitCode = WEXITSTATUS(iExitStatus);
     }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1816,19 +1938,8 @@ int             SysExec(char const * pszCommand, char const * const * pszArgs, i
     {
         int             iExitStatus = 0;
 
-        if (waitpid((pid_t) ProcessID, &iExitStatus, WUNTRACED | WNOHANG) != ProcessID)
-        {
-            if ((errno == ECHILD) || (WEXITSTATUS(iExitStatus) < 0))
-            {
-                ErrSetErrorCode(ERR_PROCESS_EXECUTE);
-                return (ERR_PROCESS_EXECUTE);
-            }
-
-            if (SysWaitPID(ProcessID, &iExitStatus, iWaitTimeout) < 0)
-                return (ErrGetErrorCode());
-        }
-        else
-            iExitStatus = WEXITSTATUS(iExitStatus);
+        if (SysWaitPID(ProcessID, &iExitStatus, iWaitTimeout) < 0)
+            return (ErrGetErrorCode());
 
         if (piExitStatus != NULL)
             *piExitStatus = iExitStatus;
@@ -1864,7 +1975,6 @@ void            SysSetBreakHandler(void (*BreakHandler) (void))
 //  Setup signal handlers and enable signals
 ///////////////////////////////////////////////////////////////////////////////
     signal(SIGINT, SysBreakHandlerRoutine);
-    signal(SIGQUIT, SysBreakHandlerRoutine);
     signal(SIGHUP, SysBreakHandlerRoutine);
 
 
@@ -1872,7 +1982,6 @@ void            SysSetBreakHandler(void (*BreakHandler) (void))
 
     sigemptyset(&SigMask);
     sigaddset(&SigMask, SIGINT);
-    sigaddset(&SigMask, SIGQUIT);
     sigaddset(&SigMask, SIGHUP);
 
     pthread_sigmask(SIG_UNBLOCK, &SigMask, NULL);
@@ -2225,7 +2334,26 @@ SYS_INT64       SysMsTime(void)
 int             SysExistFile(const char *pszFilePath)
 {
 
-    return ((access(pszFilePath, F_OK) == 0) ? 1 : 0);
+    struct stat     FS;
+
+    if (stat(pszFilePath, &FS) != 0)
+        return (0);
+
+    return ((S_ISDIR(FS.st_mode)) ? 0 : 1);
+
+}
+
+
+
+int             SysExistDir(const char *pszDirPath)
+{
+
+    struct stat     FS;
+
+    if (stat(pszDirPath, &FS) != 0)
+        return (0);
+
+    return ((S_ISDIR(FS.st_mode)) ? 1 : 0);
 
 }
 
@@ -2367,7 +2495,6 @@ int             SysGetFileInfo(char const * pszFileName, SYS_FILE_INFO & FI)
             ((S_ISDIR(stat_buffer.st_mode)) ? ftDirectory :
             ((S_ISLNK(stat_buffer.st_mode)) ? ftLink : ftOther));
     FI.ulSize = (unsigned long) stat_buffer.st_size;
-    FI.tCreat = stat_buffer.st_ctime;
     FI.tMod = stat_buffer.st_mtime;
 
     return (0);
@@ -2376,6 +2503,26 @@ int             SysGetFileInfo(char const * pszFileName, SYS_FILE_INFO & FI)
 
 
 
+int             SysSetFileModTime(char const * pszFileName, time_t tMod)
+{
+
+    struct utimbuf  TMB;
+
+    TMB.actime = tMod;
+    TMB.modtime = tMod;
+
+    if (utime(pszFileName, &TMB) != 0)
+    {
+        ErrSetErrorCode(ERR_SET_FILE_TIME);
+        return (ERR_SET_FILE_TIME);
+    }
+
+    return (0);
+
+}
+
+
+
 char           *SysStrDup(const char *pszString)
 {
 
@@ -2488,6 +2635,21 @@ int             SysVSNPrintf(char *pszBuffer, int iSize, char const * pszFormat,
 
 
 
+int             SysFileSync(FILE *pFile)
+{
+
+    if (fflush(pFile) || fsync(fileno(pFile)))
+    {
+        ErrSetErrorCode(ERR_FILE_WRITE);
+        return (ERR_FILE_WRITE);
+    }
+
+    return (0);
+
+}
+
+
+
 char           *SysStrTok(char *pszData, char const * pszDelim, char **ppszSavePtr)
 {
 
@@ -2533,19 +2695,61 @@ char           *SysAscTime(struct tm * pTStruct, char *pszBuffer, int iBufferSiz
 
 
 
+unsigned long   SysGetTimeZone(void)
+{
+
+    return ((unsigned long) timezone);
+
+}
+
+
+
 static SYS_SPINLOCK SysTestAndSet(SYS_SPINLOCK * pSpinLock)
 {
 
     unsigned int    uValue;
 
+#if defined(XMAIL_SPARC)
+
     __asm__ __volatile__(
-            "xchgl %0, %1"
-          : "=r"(uValue), "=m"(*pSpinLock)
-          : "0"(1), "m"(*pSpinLock)
-          : "memory");
+        "ldstub %1,%0;\n":
+        "=r"(uValue), "=m"(*pSpinLock):
+        "m"(*pSpinLock));
 
     return (uValue);
 
+#elif defined(XMAIL_X86)
+
+    __asm__  __volatile__(
+        "xchgl %0, %1;\n":
+        "=r"(uValue), "=m"(*pSpinLock):
+        "0"(1), "m"(*pSpinLock):
+        "memory");
+
+    return (uValue);
+
+#elif defined(XMAIL_PPC)
+
+    __asm__ __volatile__(
+        "      sync;\n"
+        "0:    lwarx %0,0,%1;\n"
+        "      xor. %0,%3,%0;\n"
+        "      bne 1f;\n"
+        "      stwcx. %2,0,%1;\n"
+        "      bne- 0b;\n"
+        "1:    sync;\n":
+        "=&r"(uValue):
+        "r"(pSpinLock), "r"(1), "r"(0):
+        "cr0", "memory");
+
+    return ((uValue == 0) ? 1: 0);
+
+#else
+
+#error CPU type not defined
+
+#endif
+
 }
 
 
@@ -2585,3 +2789,130 @@ int             SysSpinRelease(SYS_SPINLOCK * pSpinLock)
     return (0);
 
 }
+
+
+
+int             SysGetDiskSpace(char const * pszPath, SYS_INT64 * pTotal, SYS_INT64 * pFree)
+{
+
+    struct statfs   SFS;
+	
+    if (statfs(pszPath, &SFS) != 0)
+    {
+        ErrSetErrorCode(ERR_GET_DISK_SPACE_INFO);
+        return (ERR_GET_DISK_SPACE_INFO);
+    }
+
+
+    *pTotal = (SYS_INT64) SFS.f_bsize * (SYS_INT64) SFS.f_blocks;
+
+    *pFree = (SYS_INT64) SFS.f_bsize * (SYS_INT64) SFS.f_bavail;
+
+    return (0);
+
+}
+
+
+
+int             SysMemoryInfo(SYS_INT64 * pRamTotal, SYS_INT64 * pRamFree,
+                        SYS_INT64 * pVirtTotal, SYS_INT64 * pVirtFree)
+{
+
+    struct sysinfo  SI;
+
+    if (sysinfo(&SI) < 0)
+    {
+        ErrSetErrorCode(ERR_GET_MEMORY_INFO);
+        return (ERR_GET_MEMORY_INFO);
+    }
+
+
+    *pRamTotal = (SYS_INT64) SI.totalram;
+
+    *pRamFree = (SYS_INT64) SI.freeram;
+
+    *pVirtTotal = (SYS_INT64) SI.totalswap + (SYS_INT64) SI.totalram;
+
+    *pVirtFree = (SYS_INT64) SI.freeswap + (SYS_INT64) SI.freeram;
+
+    return (0);
+
+}
+
+
+
+static unsigned int SysStkCall(unsigned int (*pProc)(void *), void * pData)
+{
+
+    srand(getpid() * (unsigned int) time(NULL) * uSRandBase);
+
+
+    unsigned int    uResult,
+                    uStkDisp = (unsigned int) (rand() % MAX_STACK_SHIFT) & ~(STACK_ALIGN_BYTES - 1);
+
+#if !defined(USE_ASM_STK_DISP)
+
+    void           *pStkSpace = alloca(uStkDisp);
+
+
+    uResult = pProc(pData);
+
+#else
+#if defined(XMAIL_SPARC)
+
+    __asm__ __volatile__(
+            "sub %%sp, %0, %%sp\n":
+            :
+            "r"(uStkDisp));
+
+
+    uResult = pProc(pData);
+
+
+    __asm__ __volatile__(
+            "add %%sp, %0, %%sp\n":
+            :
+            "r"(uStkDisp));
+
+#elif defined(XMAIL_X86)
+
+    __asm__ __volatile__(
+            "sub %0, %%esp\n":
+            :
+            "r"(uStkDisp));
+
+
+    uResult = pProc(pData);
+
+
+    __asm__ __volatile__(
+            "add %0, %%esp\n":
+            :
+            "r"(uStkDisp));
+
+#elif defined(XMAIL_PPC)
+
+    __asm__ __volatile__(
+            "subf %%r1, %0, %%r1\n":
+            :
+            "r"(uStkDisp));
+
+
+    uResult = pProc(pData);
+
+
+    __asm__ __volatile__(
+            "add %%r1, %0, %%r1\n":
+            :
+            "r"(uStkDisp));
+
+#else
+
+#error CPU type not defined
+
+#endif
+#endif
+
+    return (uResult);
+
+}
