@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,9 +40,15 @@
 #include "SysTypesSolaris.h"
 
 #else           // #ifdef __SOLARIS__
+#ifdef __FREEBSD__
+
+#include "SysTypesFreeBSD.h"
+
+#else           // #ifdef __FREEBSD__
 
 #error System type not defined
 
+#endif          // #ifdef __FREEBSD__
 #endif          // #ifdef __SOLARIS__
 #endif          // #ifdef __LINUX__
 #endif          // #ifdef WIN32
