@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,10 +30,13 @@
 #include "SList.h"
 #include "BuffSock.h"
 #include "MailConfig.h"
-#include "MailSvr.h"
-#include "MiscUtils.h"
 #include "SvrUtils.h"
 #include "UsrUtils.h"
+#include "MessQueue.h"
+#include "SMAILUtils.h"
+#include "QueueUtils.h"
+#include "MailSvr.h"
+#include "MiscUtils.h"
 #include "UsrAuth.h"
 
 
@@ -43,7 +46,6 @@
 
 
 #define USER_AUTH_DIR               "userauth"
-#define USER_AUTH_DEFAULT_CONFIG    "defaultauth.tab"
 #define AUTH_LINE_MAX               1024
 #define AUTH_AUTHENTICATE_CONFIG    "userauth"
 #define AUTH_ADD_CONFIG             "useradd"
@@ -83,16 +85,16 @@ static int      UAthMacroSubstitutes(char **ppszCmdTokens, char const * pszDomai
 
 
 
-char           *UAthGetRootPath(char const * pszService, char *pszAuthPath)
+char           *UAthGetRootPath(char const * pszService, char *pszAuthPath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszAuthPath);
+    CfgGetRootPath(pszAuthPath, iMaxPath);
 
-    strcat(pszAuthPath, USER_AUTH_DIR);
+    StrNCat(pszAuthPath, USER_AUTH_DIR, iMaxPath);
 
     AppendSlash(pszAuthPath);
 
-    strcat(pszAuthPath, pszService);
+    StrNCat(pszAuthPath, pszService, iMaxPath);
 
     AppendSlash(pszAuthPath);
 
@@ -111,7 +113,7 @@ static int      UAthGetConfigPath(char const * pszService, char const * pszDomai
 
     char            szAuthPath[SYS_MAX_PATH] = "";
 
-    UAthGetRootPath(pszService, szAuthPath);
+    UAthGetRootPath(pszService, szAuthPath, sizeof(szAuthPath));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Check domain specific config
@@ -124,7 +126,7 @@ static int      UAthGetConfigPath(char const * pszService, char const * pszDomai
 ///////////////////////////////////////////////////////////////////////////////
 //  Check default config
 ///////////////////////////////////////////////////////////////////////////////
-    sprintf(pszConfigPath, "%s%s", szAuthPath, USER_AUTH_DEFAULT_CONFIG);
+    sprintf(pszConfigPath, "%s.tab", szAuthPath);
 
     if (SysExistFile(pszConfigPath))
         return (0);
@@ -169,7 +171,7 @@ static int      UAthExecAuthOp(char const * pszService, char const * pszAuthOp,
 
         int             iFieldsCount = StrStringsCount(ppszCmdTokens);
 
-        if ((iFieldsCount > 1) && (stricmp(ppszCmdTokens[0], AUTH_ADD_CONFIG) == 0))
+        if ((iFieldsCount > 1) && (stricmp(ppszCmdTokens[0], pszAuthOp) == 0))
         {
 ///////////////////////////////////////////////////////////////////////////////
 //  Do auth line macro substitution
@@ -317,7 +319,7 @@ static int      UAthMacroSubstitutes(char **ppszCmdTokens, char const * pszDomai
         {
             char            szUserPath[SYS_MAX_PATH] = "";
 
-            UsrGetUserPath(pUI, szUserPath, 0);
+            UsrGetUserPath(pUI, szUserPath, sizeof(szUserPath), 0);
 
 
             char           *pszNewValue = SysStrDup(szUserPath);
