@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,16 +30,20 @@
 #include "SList.h"
 #include "BuffSock.h"
 #include "MailConfig.h"
+#include "UsrUtils.h"
+#include "SvrUtils.h"
+#include "MessQueue.h"
+#include "SMAILUtils.h"
+#include "QueueUtils.h"
 #include "MailSvr.h"
 #include "MiscUtils.h"
-#include "SvrUtils.h"
 #include "MailDomains.h"
 #include "POP3GwLink.h"
 #include "ExtAliases.h"
-#include "UsrUtils.h"
 #include "Maildir.h"
 #include "TabIndex.h"
 #include "SMTPUtils.h"
+#include "AliasDomain.h"
 #include "UsrAuth.h"
 
 
@@ -123,8 +127,8 @@ static int      UsrCalcAliasHash(char const * const * ppszTabTokens, int const *
                         SYS_UINT32 * puHashVal, bool bCaseSens);
 static int      UsrRebuildUsersIndexes(char const * pszUsrFilePath);
 static int      UsrRebuildAliasesIndexes(char const * pszAlsFilePath);
-static char    *UsrGetTableFilePath(char *pszUsrFilePath);
-static char    *UsrGetAliasFilePath(char *pszAlsFilePath);
+static char    *UsrGetTableFilePath(char *pszUsrFilePath, int iMaxPath);
+static char    *UsrGetAliasFilePath(char *pszAlsFilePath, int iMaxPath);
 static UserInfo *UsrGetUserFromStrings(char **ppszStrings);
 static UserInfoVar *UsrAllocVar(const char *pszName, const char *pszValue);
 static void     UsrFreeVar(UserInfoVar * pUIV);
@@ -132,7 +136,8 @@ static void     UsrFreeInfoList(HSLIST & InfoList);
 static UserInfoVar *UsrGetUserVar(HSLIST & InfoList, const char *pszName);
 static int      UsrWriteInfoList(HSLIST & InfoList, FILE * pProfileFile);
 static int      UsrLoadUserInfo(HSLIST & InfoList, unsigned int uUserID, const char *pszFilePath);
-static int      UsrLoadUserDefaultInfo(HSLIST & InfoList);
+static int      UsrGetDefaultInfoFile(char const * pszDomain, char * pszInfoFile, int iMaxPath);
+static int      UsrLoadUserDefaultInfo(HSLIST & InfoList, char const * pszDomain = NULL);
 static int      UsrAliasLookupNameLK(const char *pszAlsFilePath, const char *pszDomain,
                         const char *pszAlias, char *pszName = NULL, bool bWildMatch = true);
 static int      UsrWriteAlias(FILE * pAlsFile, AliasInfo * pAI);
@@ -140,11 +145,13 @@ static bool     UsrIsWildAlias(char const * pszAlias);
 static int      UsrRemoveUserAlias(char const * pszDomain, char const * pszName);
 static UserInfo *UsrGetUserByNameLK(const char *pszUsrFilePath, const char *pszDomain,
                         const char *pszName);
+static UserInfo *UsrGetUserByNameOrAliasNDA(const char *pszDomain, const char *pszName,
+                        char *pszRealAddr);
 static int      UsrDropUserEnv(UserInfo * pUI);
 static int      UsrWriteUser(UserInfo * pUI, FILE * pUsrFile);
 static int      UsrCreateMailbox(char const * pszUsrUserPath);
 static int      UsrPrepareUserEnv(UserInfo * pUI);
-static char    *UsrGetPop3LocksPath(UserInfo * pUI, char *pszPop3LockPath);
+static char    *UsrGetPop3LocksPath(UserInfo * pUI, char *pszPop3LockPath, int iMaxPath);
 
 
 
@@ -185,7 +192,10 @@ static int      UsrCalcAliasHash(char const * const * ppszTabTokens, int const *
 ///////////////////////////////////////////////////////////////////////////////
 //  This will group wild alias ( * ? )
 ///////////////////////////////////////////////////////////////////////////////
-    if (UsrIsWildAlias(ppszTabTokens[alsAlias]))
+    int             iFieldsCount = StrStringsCount(ppszTabTokens);
+
+    if ((iFieldsCount > alsAlias) &&
+            (UsrIsWildAlias(ppszTabTokens[alsAlias]) || UsrIsWildAlias(ppszTabTokens[alsDomain])))
     {
         *puHashVal = WILD_ALIASES_HASH;
 
@@ -204,7 +214,7 @@ int             UsrCheckUsersIndexes(void)
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Align Domain-Name index
@@ -225,7 +235,7 @@ int             UsrCheckAliasesIndexes(void)
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Align Domain-Alias index
@@ -273,12 +283,12 @@ static int      UsrRebuildAliasesIndexes(char const * pszAlsFilePath)
 
 
 
-static char    *UsrGetTableFilePath(char *pszUsrFilePath)
+static char    *UsrGetTableFilePath(char *pszUsrFilePath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszUsrFilePath);
+    CfgGetRootPath(pszUsrFilePath, iMaxPath);
 
-    strcat(pszUsrFilePath, SVR_TABLE_FILE);
+    StrNCat(pszUsrFilePath, SVR_TABLE_FILE, iMaxPath);
 
     return (pszUsrFilePath);
 
@@ -286,12 +296,12 @@ static char    *UsrGetTableFilePath(char *pszUsrFilePath)
 
 
 
-static char    *UsrGetAliasFilePath(char *pszAlsFilePath)
+static char    *UsrGetAliasFilePath(char *pszAlsFilePath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszAlsFilePath);
+    CfgGetRootPath(pszAlsFilePath, iMaxPath);
 
-    strcat(pszAlsFilePath, SVR_ALIAS_FILE);
+    StrNCat(pszAlsFilePath, SVR_ALIAS_FILE, iMaxPath);
 
     return (pszAlsFilePath);
 
@@ -299,12 +309,12 @@ static char    *UsrGetAliasFilePath(char *pszAlsFilePath)
 
 
 
-char           *UsrGetMLTableFilePath(UserInfo * pUI, char *pszMLTablePath)
+char           *UsrGetMLTableFilePath(UserInfo * pUI, char *pszMLTablePath, int iMaxPath)
 {
 
-    UsrGetUserPath(pUI, pszMLTablePath);
+    UsrGetUserPath(pUI, pszMLTablePath, iMaxPath, 1);
 
-    strcat(pszMLTablePath, MLUSERS_TABLE_FILE);
+    StrNCat(pszMLTablePath, MLUSERS_TABLE_FILE, iMaxPath);
 
     return (pszMLTablePath);
 
@@ -318,7 +328,7 @@ UserType        UsrGetUserType(UserInfo * pUI)
     if (pUI->pszType == NULL)
         return (usrTypeError);
 
-    switch (pUI->pszType[0])
+    switch (ToUpper(pUI->pszType[0]))
     {
         case ('U'):
             return (usrTypeUser);
@@ -354,7 +364,7 @@ UserInfo       *UsrCreateDefaultUser(char const * pszDomain, char const * pszNam
 ///////////////////////////////////////////////////////////////////////////////
     ListInit(pUI->InfoList);
 
-    UsrLoadUserDefaultInfo(pUI->InfoList);
+    UsrLoadUserDefaultInfo(pUI->InfoList, pszDomain);
 
     return (pUI);
 
@@ -395,9 +405,9 @@ static UserInfo *UsrGetUserFromStrings(char **ppszStrings)
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetUserPath(pUI, szUsrFilePath);
+    UsrGetUserPath(pUI, szUsrFilePath, sizeof(szUsrFilePath), 1);
 
-    strcat(szUsrFilePath, USER_PROFILE_FILE);
+    StrNCat(szUsrFilePath, USER_PROFILE_FILE, sizeof(szUsrFilePath));
 
     UsrLoadUserInfo(pUI->InfoList, pUI->uUserID, szUsrFilePath);
 
@@ -448,6 +458,38 @@ char           *UsrGetUserInfoVar(UserInfo * pUI, const char *pszName,
 
 
 
+int             UsrGetUserInfoVarInt(UserInfo * pUI, const char *pszName, int iDefault)
+{
+
+    UserInfoVar    *pUIV = UsrGetUserVar(pUI->InfoList, pszName);
+
+    return ((pUIV != NULL) ? atoi(pUIV->pszValue): iDefault);
+
+}
+
+
+
+int             UsrDelUserInfoVar(UserInfo * pUI, const char *pszName)
+{
+
+    UserInfoVar    *pUIV = UsrGetUserVar(pUI->InfoList, pszName);
+
+    if (pUIV == NULL)
+    {
+        ErrSetErrorCode(ERR_USER_VAR_NOT_FOUND);
+        return (ERR_USER_VAR_NOT_FOUND);
+    }
+
+    ListRemovePtr(pUI->InfoList, (PLISTLINK) pUIV);
+
+    UsrFreeVar(pUIV);
+
+    return (0);
+
+}
+
+
+
 int             UsrSetUserInfoVar(UserInfo * pUI, const char *pszName,
                         const char *pszValue)
 {
@@ -603,7 +645,8 @@ static int      UsrLoadUserInfo(HSLIST & InfoList, unsigned int uUserID,
 {
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(pszFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(pszFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -613,8 +656,8 @@ static int      UsrLoadUserInfo(HSLIST & InfoList, unsigned int uUserID,
 
     if (pProfileFile == NULL)
     {
-        ErrSetErrorCode(ERR_NO_USER_PRFILE);
         RLckUnlockSH(hResLock);
+        ErrSetErrorCode(ERR_NO_USER_PRFILE);
         return (ERR_NO_USER_PRFILE);
     }
 
@@ -653,18 +696,56 @@ static int      UsrLoadUserInfo(HSLIST & InfoList, unsigned int uUserID,
 
 
 
-static int      UsrLoadUserDefaultInfo(HSLIST & InfoList)
+static int      UsrGetDefaultInfoFile(char const * pszDomain, char * pszInfoFile, int iMaxPath)
+{
+
+    if (pszDomain != NULL)
+    {
+///////////////////////////////////////////////////////////////////////////////
+//  Try to lookup domain specific configuration
+///////////////////////////////////////////////////////////////////////////////
+        MDomGetDomainPath(pszDomain, pszInfoFile, iMaxPath, 1);
+
+        StrNCat(pszInfoFile, DEFAULT_USER_PROFILE_FILE, iMaxPath);
+
+
+        if (SysExistFile(pszInfoFile))
+            return (0);
+
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Try to lookup global configuration
+///////////////////////////////////////////////////////////////////////////////
+    CfgGetRootPath(pszInfoFile, iMaxPath);
+
+    StrNCat(pszInfoFile, DEFAULT_USER_PROFILE_FILE, iMaxPath);
+
+    if (!SysExistFile(pszInfoFile))
+    {
+        ErrSetErrorCode(ERR_NO_USER_DEFAULT_PRFILE);
+        return (ERR_NO_USER_DEFAULT_PRFILE);
+    }
+
+
+    return (0);
+
+}
+
+
+
+static int      UsrLoadUserDefaultInfo(HSLIST & InfoList, char const * pszDomain)
 {
 
     char            szUserDefFilePath[SYS_MAX_PATH] = "";
 
-    CfgGetRootPath(szUserDefFilePath);
-
-    strcat(szUserDefFilePath, DEFAULT_USER_PROFILE_FILE);
+    if (UsrGetDefaultInfoFile(pszDomain, szUserDefFilePath, sizeof(szUserDefFilePath)) < 0)
+        return (ErrGetErrorCode());
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUserDefFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUserDefFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -763,7 +844,8 @@ static int      UsrAliasLookupNameLK(const char *pszAlsFilePath, const char *psz
 
             int             iFieldsCount = StrStringsCount(ppszTabTokens);
 
-            if ((iFieldsCount >= alsMax) && (stricmp(pszDomain, ppszTabTokens[alsDomain]) == 0) &&
+            if ((iFieldsCount >= alsMax) &&
+                    StrIWildMatch(pszDomain, ppszTabTokens[alsDomain]) &&
                     StrIWildMatch(pszAlias, ppszTabTokens[alsAlias]))
             {
                 if (pszName != NULL)
@@ -794,11 +876,12 @@ int             UsrAliasLookupName(const char *pszDomain, const char *pszAlias,
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAlsFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAlsFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (0);
@@ -907,11 +990,12 @@ int             UsrAddAlias(AliasInfo * pAI)
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -987,7 +1071,7 @@ int             UsrRemoveAlias(const char *pszDomain, const char *pszAlias)
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -995,7 +1079,8 @@ int             UsrRemoveAlias(const char *pszDomain, const char *pszAlias)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -1107,7 +1192,7 @@ int             UsrRemoveDomainAliases(const char *pszDomain)
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -1115,7 +1200,8 @@ int             UsrRemoveDomainAliases(const char *pszDomain)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -1229,7 +1315,7 @@ static int      UsrRemoveUserAlias(char const * pszDomain, char const * pszName)
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -1237,7 +1323,8 @@ static int      UsrRemoveUserAlias(char const * pszDomain, char const * pszName)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAlsFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -1374,16 +1461,17 @@ static UserInfo *UsrGetUserByNameLK(const char *pszUsrFilePath, const char *pszD
 
 
 
-UserInfo       *UsrGetUserByName(const char *pszDomain, const char *pszName)
+UserInfo       *UsrLookupUser(const char *pszDomain, const char *pszName)
 {
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (NULL);
@@ -1400,7 +1488,27 @@ UserInfo       *UsrGetUserByName(const char *pszDomain, const char *pszName)
 
 
 
-UserInfo       *UsrGetUserByNameOrAlias(const char *pszDomain, const char *pszName,
+UserInfo       *UsrGetUserByName(const char *pszDomain, const char *pszName)
+{
+///////////////////////////////////////////////////////////////////////////////
+//  Check for alias domain
+///////////////////////////////////////////////////////////////////////////////
+    UserInfo       *pUI = UsrLookupUser(pszDomain, pszName);
+    char            szADomain[MAX_HOST_NAME] = "";
+
+///////////////////////////////////////////////////////////////////////////////
+//  Check for alias domain if first lookup failed
+///////////////////////////////////////////////////////////////////////////////
+    if ((pUI == NULL) && ADomLookupDomain(pszDomain, szADomain, true))
+        pUI = UsrLookupUser(szADomain, pszName);
+
+    return (pUI);
+
+}
+
+
+
+static UserInfo    *UsrGetUserByNameOrAliasNDA(const char *pszDomain, const char *pszName,
                         char *pszRealAddr)
 {
 
@@ -1427,11 +1535,12 @@ UserInfo       *UsrGetUserByNameOrAlias(const char *pszDomain, const char *pszNa
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (NULL);
@@ -1454,13 +1563,32 @@ UserInfo       *UsrGetUserByNameOrAlias(const char *pszDomain, const char *pszNa
 
 
 
+UserInfo       *UsrGetUserByNameOrAlias(const char *pszDomain, const char *pszName,
+                        char *pszRealAddr)
+{
+
+    UserInfo       *pUI = UsrGetUserByNameOrAliasNDA(pszDomain, pszName, pszRealAddr);
+    char            szADomain[MAX_HOST_NAME] = "";
+
+///////////////////////////////////////////////////////////////////////////////
+//  Check for alias domain if first lookup failed
+///////////////////////////////////////////////////////////////////////////////
+    if ((pUI == NULL) && ADomLookupDomain(pszDomain, szADomain, true))
+        pUI = UsrGetUserByNameOrAliasNDA(szADomain, pszName, pszRealAddr);
+
+    return (pUI);
+
+}
+
+
+
 int             UsrRemoveUser(const char *pszDomain, const char *pszName,
                         unsigned int uUserID)
 {
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -1468,7 +1596,8 @@ int             UsrRemoveUser(const char *pszDomain, const char *pszName,
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -1607,7 +1736,7 @@ int             UsrModifyUser(UserInfo * pUI)
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -1615,7 +1744,8 @@ int             UsrModifyUser(UserInfo * pUI)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -1759,7 +1889,7 @@ int             UsrRemoveDomainUsers(const char *pszDomain)
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -1767,7 +1897,8 @@ int             UsrRemoveDomainUsers(const char *pszDomain)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -1884,7 +2015,7 @@ static int      UsrDropUserEnv(UserInfo * pUI)
 ///////////////////////////////////////////////////////////////////////////////
     char            szUsrUserPath[SYS_MAX_PATH] = "";
 
-    UsrGetUserPath(pUI, szUsrUserPath, 0);
+    UsrGetUserPath(pUI, szUsrUserPath, sizeof(szUsrUserPath), 0);
 
     if (MscClearDirectory(szUsrUserPath) < 0)
         return (ErrGetErrorCode());
@@ -1995,11 +2126,12 @@ int             UsrAddUser(UserInfo * pUI)
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2110,10 +2242,10 @@ static int      UsrCreateMailbox(char const * pszUsrUserPath)
 ///////////////////////////////////////////////////////////////////////////////
     char            szUsrMailboxPath[SYS_MAX_PATH] = "";
 
-    strcpy(szUsrMailboxPath, pszUsrUserPath);
+    StrSNCpy(szUsrMailboxPath, pszUsrUserPath);
 
     AppendSlash(szUsrMailboxPath);
-    strcat(szUsrMailboxPath, USER_MAILBOX_DIR);
+    StrSNCat(szUsrMailboxPath, USER_MAILBOX_DIR);
 
     if (SysMakeDir(szUsrMailboxPath) < 0)
         return (ErrGetErrorCode());
@@ -2131,7 +2263,7 @@ static int      UsrPrepareUserEnv(UserInfo * pUI)
 
     char            szUsrUserPath[SYS_MAX_PATH] = "";
 
-    UsrGetUserPath(pUI, szUsrUserPath, 0);
+    UsrGetUserPath(pUI, szUsrUserPath, sizeof(szUsrUserPath), 0);
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Create main directory
@@ -2159,10 +2291,10 @@ static int      UsrPrepareUserEnv(UserInfo * pUI)
 ///////////////////////////////////////////////////////////////////////////////
         char            szMLUsersFilePath[SYS_MAX_PATH] = "";
 
-        strcpy(szMLUsersFilePath, szUsrUserPath);
+        StrSNCpy(szMLUsersFilePath, szUsrUserPath);
 
         AppendSlash(szMLUsersFilePath);
-        strcat(szMLUsersFilePath, MLUSERS_TABLE_FILE);
+        StrSNCat(szMLUsersFilePath, MLUSERS_TABLE_FILE);
 
         if (MscCreateEmptyFile(szMLUsersFilePath) < 0)
         {
@@ -2178,10 +2310,10 @@ static int      UsrPrepareUserEnv(UserInfo * pUI)
 ///////////////////////////////////////////////////////////////////////////////
     char            szUsrProfileFilePath[SYS_MAX_PATH] = "";
 
-    strcpy(szUsrProfileFilePath, szUsrUserPath);
+    StrSNCpy(szUsrProfileFilePath, szUsrUserPath);
 
     AppendSlash(szUsrProfileFilePath);
-    strcat(szUsrProfileFilePath, USER_PROFILE_FILE);
+    StrSNCat(szUsrProfileFilePath, USER_PROFILE_FILE);
 
     FILE           *pProfileFile = fopen(szUsrProfileFilePath, "wt");
 
@@ -2211,21 +2343,22 @@ int             UsrFlushUserVars(UserInfo * pUI)
 
     char            szUsrUserPath[SYS_MAX_PATH] = "";
 
-    UsrGetUserPath(pUI, szUsrUserPath, 0);
+    UsrGetUserPath(pUI, szUsrUserPath, sizeof(szUsrUserPath), 0);
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Build profile file path
 ///////////////////////////////////////////////////////////////////////////////
     char            szUsrProfileFilePath[SYS_MAX_PATH] = "";
 
-    strcpy(szUsrProfileFilePath, szUsrUserPath);
+    StrSNCpy(szUsrProfileFilePath, szUsrUserPath);
 
     AppendSlash(szUsrProfileFilePath);
-    strcat(szUsrProfileFilePath, USER_PROFILE_FILE);
+    StrSNCat(szUsrProfileFilePath, USER_PROFILE_FILE);
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrProfileFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szUsrProfileFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2260,11 +2393,12 @@ int             UsrGetDBFileSnapShot(const char *pszFileName)
 
     char            szUsrFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetTableFilePath(szUsrFilePath);
+    UsrGetTableFilePath(szUsrFilePath, sizeof(szUsrFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUsrFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szUsrFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2393,19 +2527,19 @@ UserInfo       *UsrGetNextUser(USRF_HANDLE hUsersDB)
 
 
 
-static char    *UsrGetPop3LocksPath(UserInfo * pUI, char *pszPop3LockPath)
+static char    *UsrGetPop3LocksPath(UserInfo * pUI, char *pszPop3LockPath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszPop3LockPath);
+    CfgGetRootPath(pszPop3LockPath, iMaxPath);
 
-    strcat(pszPop3LockPath, POP3_LOCKS_DIR);
+    StrNCat(pszPop3LockPath, POP3_LOCKS_DIR, iMaxPath);
     AppendSlash(pszPop3LockPath);
 
     char            szUserAddress[MAX_ADDR_NAME] = "";
 
     UsrGetAddress(pUI, szUserAddress);
 
-    strcat(pszPop3LockPath, szUserAddress);
+    StrNCat(pszPop3LockPath, szUserAddress, iMaxPath);
 
     return (pszPop3LockPath);
 
@@ -2418,7 +2552,7 @@ int             UsrPOP3Lock(UserInfo * pUI)
 
     char            szLockPath[SYS_MAX_PATH] = "";
 
-    UsrGetPop3LocksPath(pUI, szLockPath);
+    UsrGetPop3LocksPath(pUI, szLockPath, sizeof(szLockPath));
 
     if (SysLockFile(szLockPath) < 0)
         return (ErrGetErrorCode());
@@ -2434,7 +2568,7 @@ void            UsrPOP3Unlock(UserInfo * pUI)
 
     char            szLockPath[SYS_MAX_PATH] = "";
 
-    UsrGetPop3LocksPath(pUI, szLockPath);
+    UsrGetPop3LocksPath(pUI, szLockPath, sizeof(szLockPath));
 
     SysUnlockFile(szLockPath);
 
@@ -2447,9 +2581,9 @@ int             UsrClearPop3LocksDir(void)
 
     char            szLocksDir[SYS_MAX_PATH] = "";
 
-    CfgGetRootPath(szLocksDir);
+    CfgGetRootPath(szLocksDir, sizeof(szLocksDir));
 
-    strcat(szLocksDir, POP3_LOCKS_DIR);
+    StrNCat(szLocksDir, POP3_LOCKS_DIR, sizeof(szLocksDir));
 
     return (MscClearDirectory(szLocksDir));
 
@@ -2457,12 +2591,12 @@ int             UsrClearPop3LocksDir(void)
 
 
 
-char           *UsrGetUserPath(UserInfo * pUI, char *pszUserPath, int iFinalSlash)
+char           *UsrGetUserPath(UserInfo * pUI, char *pszUserPath, int iMaxPath, int iFinalSlash)
 {
 
-    MDomGetDomainPath(pUI->pszDomain, pszUserPath);
+    MDomGetDomainPath(pUI->pszDomain, pszUserPath, iMaxPath, 1);
 
-    strcat(pszUserPath, pUI->pszPath);
+    StrNCat(pszUserPath, pUI->pszPath, iMaxPath);
 
     if (iFinalSlash)
         AppendSlash(pszUserPath);
@@ -2473,12 +2607,12 @@ char           *UsrGetUserPath(UserInfo * pUI, char *pszUserPath, int iFinalSlas
 
 
 
-char           *UsrGetMailboxPath(UserInfo * pUI, char *pszMBPath, int iFinalSlash)
+char           *UsrGetMailboxPath(UserInfo * pUI, char *pszMBPath, int iMaxPath, int iFinalSlash)
 {
 
-    UsrGetUserPath(pUI, pszMBPath);
+    UsrGetUserPath(pUI, pszMBPath, iMaxPath, 1);
 
-    strcat(pszMBPath, USER_MAILBOX_DIR);
+    StrNCat(pszMBPath, USER_MAILBOX_DIR, iMaxPath);
     if (iFinalSlash)
         AppendSlash(pszMBPath);
 
@@ -2498,11 +2632,12 @@ int             UsrMoveToMailBox(UserInfo * pUI, char const * pszFileName,
 ///////////////////////////////////////////////////////////////////////////////
     char            szMBPath[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pUI, szMBPath, 0);
+    UsrGetMailboxPath(pUI, szMBPath, sizeof(szMBPath), 0);
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2526,13 +2661,14 @@ int             UsrMoveToMailBox(UserInfo * pUI, char const * pszFileName,
     char            szMBPath[SYS_MAX_PATH] = "",
                     szMBFile[SYS_MAX_PATH] = "";
 
-    UsrGetMailboxPath(pUI, szMBPath, 0);
+    UsrGetMailboxPath(pUI, szMBPath, sizeof(szMBPath), 0);
 
     sprintf(szMBFile, "%s" SYS_SLASH_STR "%s", szMBPath, pszMessageID);
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMBPath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2559,10 +2695,10 @@ int             UsrGetMailProcessFile(UserInfo * pUI, char *pszMPPath)
 
     char            szMPFilePath[SYS_MAX_PATH] = "";
 
-    if (UsrGetUserPath(pUI, szMPFilePath) == NULL)
+    if (UsrGetUserPath(pUI, szMPFilePath, sizeof(szMPFilePath), 1) == NULL)
         return (ErrGetErrorCode());
 
-    strcat(szMPFilePath, MAILPROCESS_FILE);
+    StrNCat(szMPFilePath, MAILPROCESS_FILE, sizeof(szMPFilePath));
 
     if (!SysExistFile(szMPFilePath))
     {
@@ -2572,7 +2708,8 @@ int             UsrGetMailProcessFile(UserInfo * pUI, char *pszMPPath)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szMPFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szMPFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2600,14 +2737,15 @@ int             UsrSetMailProcessFile(UserInfo * pUI, char const * pszMPPath)
 
     char            szMPFilePath[SYS_MAX_PATH] = "";
 
-    if (UsrGetUserPath(pUI, szMPFilePath) == NULL)
+    if (UsrGetUserPath(pUI, szMPFilePath, sizeof(szMPFilePath), 1) == NULL)
         return (ErrGetErrorCode());
 
-    strcat(szMPFilePath, MAILPROCESS_FILE);
+    StrNCat(szMPFilePath, MAILPROCESS_FILE, sizeof(szMPFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMPFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szMPFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -2650,11 +2788,12 @@ int             UsrGetAliasDBFileSnapShot(char const * pszFileName)
 
     char            szAlsFilePath[SYS_MAX_PATH] = "";
 
-    UsrGetAliasFilePath(szAlsFilePath);
+    UsrGetAliasFilePath(szAlsFilePath, sizeof(szAlsFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAlsFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAlsFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
